@@ -1,38 +1,34 @@
 /*
- *  $Id: sctpmultiserver.cc 183 2003-08-19 19:28:34Z tuexen $
+ *  $Id$
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: SCTP Multi Server
@@ -50,6 +46,7 @@
 
 
 #include "tdsystem.h"
+#include "tools.h"
 #include "thread.h"
 #include "tdsocket.h"
 #include "tdmessage.h"
@@ -96,19 +93,19 @@ class ServerSet : public Thread
    // ====== Protected data =================================================
    protected:
    void        killMe();
-   static void garbageCollection(set<ServerSet*>& gcSet);
+   static void garbageCollection(std::set<ServerSet*>& gcSet);
 
-   Socket*                ServerSocket;
-   static Synchronizable  ServersSynchronizable;
-   static set<ServerSet*> Servers;
-   static set<ServerSet*> DeletedServers;
+   Socket*                     ServerSocket;
+   static Synchronizable       ServersSynchronizable;
+   static std::set<ServerSet*> Servers;
+   static std::set<ServerSet*> DeletedServers;
 };
 
 
 // ###### Static variables ##################################################
-Synchronizable  ServerSet::ServersSynchronizable("ServerSet::ServersSynchronizable");
-set<ServerSet*> ServerSet::Servers;
-set<ServerSet*> ServerSet::DeletedServers;
+Synchronizable       ServerSet::ServersSynchronizable("ServerSet::ServersSynchronizable");
+std::set<ServerSet*> ServerSet::Servers;
+std::set<ServerSet*> ServerSet::DeletedServers;
 
 
 // ###### Constructor #######################################################
@@ -144,10 +141,10 @@ void ServerSet::killMe()
 
 
 // ###### Do garbage collection #############################################
-void ServerSet::garbageCollection(set<ServerSet*>& gcSet)
+void ServerSet::garbageCollection(std::set<ServerSet*>& gcSet)
 {
    ServerSet::ServersSynchronizable.synchronized();
-   set<ServerSet*>::iterator iterator = gcSet.begin();
+   std::set<ServerSet*>::iterator iterator = gcSet.begin();
    while(iterator != gcSet.end()) {
       ServerSet* server = *iterator;
       delete server;
@@ -202,26 +199,26 @@ EchoServer::EchoServer(Socket* socket, const cardinal unreliable, const bool dis
 // ###### Main loop #########################################################
 void EchoServer::run()
 {
-   cardinal inStreams  = 1;
+   // cardinal inStreams  = 1;
    cardinal outStreams = 1;
    bool     shutdown   = false;
    char dataBuffer[16384 + 1];
    while(!shutdown) {
       // ====== Receive string from association =============================
-      SocketMessage<CSpace(sizeof(sctp_sndrcvinfo))> message;
+      SocketMessage<sizeof(sctp_sndrcvinfo)> message;
       message.setBuffer((char*)&dataBuffer,sizeof(dataBuffer));
 
       // ====== Receive string from association =============================
       const ssize_t dataLength = ServerSocket->receiveMsg(&message.Header,0);
       if(dataLength <= 0) {
-         printTimeStamp(cout);
+         printTimeStamp(std::cout);
          if(dataLength < 0) {
-            cout << "Thread shutdown due to receive error #"
-                 << -dataLength << "." << endl
-                 << "   Description: " << strerror(-dataLength) << "!" << endl;
+            std::cout << "Thread shutdown due to receive error #"
+                 << -dataLength << "." << std::endl
+                 << "   Description: " << strerror(-dataLength) << "!" << std::endl;
          }
          else {
-            cout << "Thread shutdown!" << endl;
+            std::cout << "Thread shutdown!" << std::endl;
          }
          killMe();
          return;
@@ -237,10 +234,10 @@ void EchoServer::run()
          while(cmsg != NULL) {
             if((cmsg->cmsg_level == IPPROTO_SCTP) &&
                (cmsg->cmsg_type  == SCTP_SNDRCV)) {
-               sctp_sndrcvinfo* info = (sctp_sndrcvinfo*)CData(cmsg);
+               const sctp_sndrcvinfo* info = (const sctp_sndrcvinfo*)CData(cmsg);
                assocID  = info->sinfo_assoc_id;
                streamID = info->sinfo_stream;
-               protoID  = info->sinfo_ppid;
+               protoID  = ntohl(info->sinfo_ppid);
             }
             cmsg = message.getNextHeader(cmsg);
          }
@@ -256,7 +253,7 @@ void EchoServer::run()
             sctp_sndrcvinfo* info = (sctp_sndrcvinfo*)message.addHeader(sizeof(sctp_sndrcvinfo),IPPROTO_SCTP,SCTP_SNDRCV);
             info->sinfo_assoc_id  = 0;
             info->sinfo_stream    = (streamID % outStreams);
-            info->sinfo_ppid      = protoID;
+            info->sinfo_ppid      = htonl(protoID);
             if((Unreliable > 0) && (info->sinfo_stream <= Unreliable - 1)) {
                info->sinfo_flags      = MSG_PR_SCTP_TTL;
                info->sinfo_timetolive = 0;
@@ -267,9 +264,9 @@ void EchoServer::run()
             }
             const ssize_t result = ServerSocket->sendMsg(&message.Header,0);
             if(result < 0) {
-               printTimeStamp(cout);
-               cout << "Thread shutdown due to send error #"
-                    << -dataLength << "!" << endl;
+               printTimeStamp(std::cout);
+               std::cout << "Thread shutdown due to send error #"
+                    << -dataLength << "!" << std::endl;
                killMe();
                return;
             }
@@ -277,17 +274,34 @@ void EchoServer::run()
 
          // ====== Print information ========================================
          char str[16384 + 256];
+         char outputDataBuffer[dataLength];
          dataBuffer[dataLength - 1] = 0x00;
+         for(size_t i = 0;i < (size_t)dataLength - 1;i++) {
+            if(dataBuffer[i] >= ' ') {
+               outputDataBuffer[i] = dataBuffer[i];
+            }
+            else {
+               outputDataBuffer[i] = '.';
+            }
+         }
+         for(int i = dataLength - 1;i >= 0;i--) {
+            if(dataBuffer[i] < ' ') {
+               outputDataBuffer[i] = 0x00;
+            }
+            else {
+               break;
+            }
+         }
          snprintf((char*)&str,sizeof(str),"#%d.%d/$%08x: %s [%s]",
                      assocID, streamID, protoID,
                      (DiscardMode == true) ? "Discard" : "Echo",
-                     dataBuffer);
+                     outputDataBuffer);
          if(ColorMode) {
-            cout << "\x1b[" << getANSIColor(streamID + 1) << "m";
+            std::cout << "\x1b[" << getANSIColor(streamID + 1) << "m";
          }
-         cout << str << endl;
+         std::cout << str << std::endl;
          if(ColorMode) {
-            cout << "\x1b[" << getANSIColor(0) << "m";
+            std::cout << "\x1b[" << getANSIColor(0) << "m";
          }
       }
 
@@ -297,7 +311,7 @@ void EchoServer::run()
          printNotification(notification);
          if(notification->sn_header.sn_type == SCTP_ASSOC_CHANGE) {
             const sctp_assoc_change* sac = &notification->sn_assoc_change;
-            inStreams  = sac->sac_inbound_streams;
+            // inStreams  = sac->sac_inbound_streams;
             outStreams = sac->sac_outbound_streams;
          }
          else if(notification->sn_header.sn_type == SCTP_SHUTDOWN_EVENT) {
@@ -305,8 +319,8 @@ void EchoServer::run()
          }
       }
    }
-   printTimeStamp(cout);
-   cout << "Thread shutdown!" << endl;
+   printTimeStamp(std::cout);
+   std::cout << "Thread shutdown!" << std::endl;
    killMe();
 }
 
@@ -339,13 +353,13 @@ CharGenServer::CharGenServer(Socket* socket, const cardinal unreliable)
 // ###### Main loop #########################################################
 void CharGenServer::run()
 {
-   SocketMessage<CSpace(sizeof(sctp_sndrcvinfo))> message;
+   SocketMessage<sizeof(sctp_sndrcvinfo)> message;
    char   dataBuffer[512];
    card64 line = 1;
 
    for(;;) {
       // ====== Generate string =============================================
-      snprintf((char*)&dataBuffer,256,"#%08Ld: ",line++);
+      snprintf((char*)&dataBuffer,256,"#%08llu: ",(unsigned long long)line++);
       integer i,j;
       j = strlen(dataBuffer);
       for(i = 30;i < 128;i++) {
@@ -373,9 +387,9 @@ void CharGenServer::run()
       }
       const ssize_t result = ServerSocket->sendMsg(&message.Header,0);
       if(result < 0) {
-         printTimeStamp(cout);
-         cout << "Thread shutdown due to send error #"
-              << -result << "!" << endl;
+         printTimeStamp(std::cout);
+         std::cout << "Thread shutdown due to send error #"
+              << -result << "!" << std::endl;
          killMe();
          return;
       }
@@ -459,8 +473,8 @@ ssize_t TrivialFileTransferServer::sendAck(const char code)
 // ###### Main loop #########################################################
 void TrivialFileTransferServer::run()
 {
-   ssize_t result;
-   result = ServerSocket->receive((char*)&Packet,sizeof(Packet));
+   integer flags  = 0;
+   ssize_t result = ServerSocket->receive((char*)&Packet,sizeof(Packet),flags);
    if(result >= 3) {
       // ====== Read Request ================================================
       if(Packet.Type == TFTP_RRQ) {
@@ -499,7 +513,8 @@ void TrivialFileTransferServer::run()
                result = sendAck(0);
                if(result > 0) {
                   // ====== Write loop ======================================
-                  result = ServerSocket->receive((char*)&Packet,sizeof(Packet));
+                  flags  = 0;
+                  result = ServerSocket->receive((char*)&Packet,sizeof(Packet),flags);
                   while(result >= 2) {
                      if(Packet.Type != TFTP_DAT) {
                         sendError(0,"Bad request!");
@@ -514,7 +529,8 @@ void TrivialFileTransferServer::run()
                         sendAck(Packet.Code);
                         break;
                      }
-                     result = ServerSocket->receive((char*)&Packet,sizeof(Packet));
+                     flags  = 0;
+                     result = ServerSocket->receive((char*)&Packet,sizeof(Packet),flags);
                   }
                }
                fclose(out);
@@ -535,13 +551,13 @@ void TrivialFileTransferServer::run()
    }
 
    if(result < 0) {
-      printTimeStamp(cout);
-      cout << "Thread shutdown due to send error #"
-           << -result << "!" << endl;
+      printTimeStamp(std::cout);
+      std::cout << "Thread shutdown due to send error #"
+                << -result << "!" << std::endl;
    }
    else {
-      printTimeStamp(cout);
-      cout << "Thread shutdown!" << endl;
+      printTimeStamp(std::cout);
+      std::cout << "Thread shutdown!" << std::endl;
    }
 
    killMe();
@@ -559,8 +575,6 @@ class DaytimeServer : public ServerSet
    // ====== Private data ===================================================
    private:
    void run();
-
-   bool TestMode;
 };
 
 
@@ -589,9 +603,9 @@ void DaytimeServer::run()
 
    const int result = ServerSocket->send(str1,strlen(str1),0);
    if(result < 0) {
-      printTimeStamp(cout);
-      cout << "Daytime server got send error #"
-           << -result << "!" << endl;
+      printTimeStamp(std::cout);
+      std::cout << "Daytime server got send error #"
+                << -result << "!" << std::endl;
    }
 
    killMe();
@@ -677,7 +691,7 @@ bool MultiServer::init(SocketAddress** addressArray,
    }
    ServerSocket = new Socket(Socket::IP,Socket::Stream,Socket::SCTP);
    if(ServerSocket == NULL) {
-      cerr << "ERROR: MultiServer::init() - Out of memory!" << endl;
+      std::cerr << "ERROR: MultiServer::init() - Out of memory!" << std::endl;
       return(false);
    }
 
@@ -688,7 +702,7 @@ bool MultiServer::init(SocketAddress** addressArray,
    init.sinit_max_attempts   = 0;
    init.sinit_max_init_timeo = 60;
    if(ServerSocket->setSocketOption(IPPROTO_SCTP,SCTP_INITMSG,(char*)&init,sizeof(init)) < 0) {
-      cerr << "ERROR: MultiServer::init() - Unable to set SCTP_INITMSG parameters!" << endl;
+      std::cerr << "ERROR: MultiServer::init() - Unable to set SCTP_INITMSG parameters!" << std::endl;
       delete ServerSocket;
       return(false);
    }
@@ -698,7 +712,7 @@ bool MultiServer::init(SocketAddress** addressArray,
    if(ServerSocket->bindx((const SocketAddress**)&socketAddressArray,
                           addresses,
                           SCTP_BINDX_ADD_ADDR) == false) {
-      cerr << "ERROR: MultiServer::init() - Unable to bind socket!" << endl;
+      std::cerr << "ERROR: MultiServer::init() - Unable to bind socket!" << std::endl;
       delete ServerSocket;
       return(false);
    }
@@ -706,7 +720,7 @@ bool MultiServer::init(SocketAddress** addressArray,
    sctp_event_subscribe events;
    memset((char*)&events,1,sizeof(events));
    if(ServerSocket->setSocketOption(IPPROTO_SCTP,SCTP_EVENTS,&events,sizeof(events)) < 0) {
-      cerr << "ERROR: MultiServer::init() - SCTP_EVENTS failed!" << endl;
+      std::cerr << "ERROR: MultiServer::init() - SCTP_EVENTS failed!" << std::endl;
       delete ServerSocket;
       return(false);
    }
@@ -729,23 +743,23 @@ void MultiServer::run()
             sctp_event_subscribe events;
             memset((char*)&events,1,sizeof(events));
             if(newSocket->setSocketOption(IPPROTO_SCTP,SCTP_EVENTS,&events,sizeof(events)) < 0) {
-               cerr << "WARNING: MultiServer::run() - SCTP_EVENTS failed!" << endl;
+               std::cerr << "WARNING: MultiServer::run() - SCTP_EVENTS failed!" << std::endl;
             }
 
-            printTimeStamp(cout);
+            printTimeStamp(std::cout);
             if(peer != NULL) {
                peer->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy);
-               cout << "Accepted association from " << *peer << " to ";
+               std::cout << "Accepted association from " << *peer << " to ";
                delete peer;
             }
             else {
-               cout << "Accepted association from (Unknown) to ";
+               std::cout << "Accepted association from (Unknown) to ";
             }
 
             switch(Type) {
                case MST_Echo:
                   {
-                     cout << "Echo server." << endl;
+                     std::cout << "Echo server." << std::endl;
                      EchoServer* server = new EchoServer(newSocket,Unreliable,false);
                      if(server == NULL) {
                         delete newSocket;
@@ -754,7 +768,7 @@ void MultiServer::run()
                 break;
                case MST_Discard:
                   {
-                     cout << "Discard server." << endl;
+                     std::cout << "Discard server." << std::endl;
                      EchoServer* server = new EchoServer(newSocket,0,true);
                      if(server == NULL) {
                         delete newSocket;
@@ -763,7 +777,7 @@ void MultiServer::run()
                 break;
                case MST_CharGen:
                   {
-                     cout << "CharGen server." << endl;
+                     std::cout << "CharGen server." << std::endl;
                      CharGenServer* server = new CharGenServer(newSocket,Unreliable);
                      if(server == NULL) {
                         delete newSocket;
@@ -772,7 +786,7 @@ void MultiServer::run()
                 break;
                case MST_TFTP:
                   {
-                     cout << "TFTP server." << endl;
+                     std::cout << "TFTP server." << std::endl;
                      TrivialFileTransferServer* server = new TrivialFileTransferServer(newSocket);
                      if(server == NULL) {
                         delete newSocket;
@@ -780,14 +794,14 @@ void MultiServer::run()
                   }
                 break;
                case MST_Test: {
-                     cout << "Test server." << endl;
+                     std::cout << "Test server." << std::endl;
                      delete newSocket;
                   }
                 break;
                case MST_Daytime:
                default:
                   {
-                     cout << "Daytime server." << endl;
+                     std::cout << "Daytime server." << std::endl;
                      DaytimeServer* server = new DaytimeServer(newSocket);
                      if(server == NULL) {
                         delete newSocket;
@@ -816,13 +830,13 @@ int main(int argc, char** argv)
    PrintNotifications = false;
    if(!sctp_isavailable()) {
 #ifdef HAVE_KERNEL_SCTP
-      cerr << "ERROR: Kernel-based SCTP is not available!" << endl;
+      std::cerr << "ERROR: Kernel-based SCTP is not available!" << std::endl;
 #else
      if(getuid() != 0) {
-        cerr << "ERROR: You need root permissions to use the SCTP Library!" << endl;
+        std::cerr << "ERROR: You need root permissions to use the SCTP Library!" << std::endl;
      }
      else {
-        cerr << "ERROR: SCTP is not available!" << endl;
+        std::cerr << "ERROR: SCTP is not available!" << std::endl;
      }
 #endif
       exit(1);
@@ -856,7 +870,7 @@ int main(int argc, char** argv)
             if(localAddressArray == NULL) {
                localAddressArray = SocketAddress::newAddressList(SCTP_MAXADDRESSES);
                if(localAddressArray == NULL) {
-                  cerr << "ERROR: Out of memory!" << endl;
+                  std::cerr << "ERROR: Out of memory!" << std::endl;
                   exit(1);
                }
             }
@@ -864,13 +878,13 @@ int main(int argc, char** argv)
                                                    SocketAddress::PF_HidePort,
                                                    &argv[i][7]);
             if(localAddressArray[localAddresses] == NULL) {
-               cerr << "ERROR: Argument #" << i << " is an invalid address!" << endl;
+               std::cerr << "ERROR: Argument #" << i << " is an invalid address!" << std::endl;
                exit(1);
             }
             localAddresses++;
          }
          else {
-            cerr << "ERROR: Too many local addresses!" << endl;
+            std::cerr << "ERROR: Too many local addresses!" << std::endl;
             exit(1);
          }
       }
@@ -896,9 +910,9 @@ int main(int argc, char** argv)
          PrintNotifications = false;
       }
       else {
-         cerr << "Usage: " << argv[0] << " "
-                 "{-force-ipv4|-use-ipv6} {-local=address1} ... {-local=addressN}  {-nocolor} {-color} {-control|-nocontrol} {-notif|-nonotif}"
-              << endl;
+         std::cerr << "Usage: " << argv[0] << " "
+                      "{-force-ipv4|-use-ipv6} {-local=address1} ... {-local=addressN}  {-nocolor} {-color} {-control|-nocontrol} {-notif|-nonotif}"
+              << std::endl;
          exit(1);
       }
    }
@@ -910,32 +924,31 @@ int main(int argc, char** argv)
             localAddressArray,
             localAddresses,
             Socket::GLAF_HideBroadcast|Socket::GLAF_HideMulticast|Socket::GLAF_HideAnycast)) {
-         cerr << "ERROR: Cannot obtain local addresses!" << endl;
+         std::cerr << "ERROR: Cannot obtain local addresses!" << std::endl;
          exit(1);
       }
       if(localAddresses < 1) {
-         cerr << "ERROR: No valid local addresses have been found?!" << endl
-              << "       Check your network interface configuration!" << endl;
+         std::cerr << "ERROR: No valid local addresses have been found?!" << std::endl
+                   << "       Check your network interface configuration!" << std::endl;
          exit(1);
       }
    }
-   unreliable = min(maxOutgoing,unreliable);
+   unreliable = std::min(maxOutgoing,unreliable);
 
 
    // ====== Print information ==============================================
-   cout << "SCTP Multi Server - Copyright (C) 2001-2003 Thomas Dreibholz" << endl;
-   cout << "------------------------------------------------------------" << endl;
-   cout << "Version:           " << __DATE__ << ", " << __TIME__ << endl;
+   std::cout << "SCTP Multi Server - Copyright (C) 2001-2026 Thomas Dreibholz" << std::endl;
+   std::cout << "------------------------------------------------------------" << std::endl;
    localAddressArray[0]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
-   cout << "Local Addresses:   " << *(localAddressArray[0]) << endl;
+   std::cout << "Local Addresses:   " << *(localAddressArray[0]) << std::endl;
    for(cardinal i = 1;i < localAddresses;i++) {
       localAddressArray[i]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
-      cout << "                   " << *(localAddressArray[i]) << endl;
+      std::cout << "                   " << *(localAddressArray[i]) << std::endl;
    }
-   cout << "Max Incoming:      " << maxIncoming << endl;
-   cout << "Max Outgoing:      " << maxOutgoing << endl;
-   cout << "Unreliable:        " << unreliable     << endl;
-   cout << endl << endl;
+   std::cout << "Max Incoming:      " << maxIncoming << std::endl;
+   std::cout << "Max Outgoing:      " << maxOutgoing << std::endl;
+   std::cout << "Unreliable:        " << unreliable     << std::endl;
+   std::cout << std::endl << std::endl;
 
 
    // ====== Execute main program ===========================================
@@ -944,7 +957,7 @@ int main(int argc, char** argv)
    if((echoServer == NULL) ||
       (echoServer->init(localAddressArray,localAddresses,
                         ECHO_PORT,MultiServer::MST_Echo,maxIncoming,maxOutgoing,unreliable) == false)) {
-      cerr << "ERROR: Unable to initialize Echo server!" << endl;
+      std::cerr << "ERROR: Unable to initialize Echo server!" << std::endl;
       exit(1);
    }
 #endif
@@ -954,7 +967,7 @@ int main(int argc, char** argv)
    if((discardServer == NULL) ||
       (discardServer->init(localAddressArray,localAddresses,
                            DISCARD_PORT,MultiServer::MST_Discard,maxIncoming,1,unreliable) == false)) {
-      cerr << "ERROR: Unable to initialize Discard server!" << endl;
+      std::cerr << "ERROR: Unable to initialize Discard server!" << std::endl;
       exit(1);
    }
 #endif
@@ -964,7 +977,7 @@ int main(int argc, char** argv)
    if((daytimeServer == NULL) ||
       (daytimeServer->init(localAddressArray,localAddresses,
                            DAYTIME_PORT,MultiServer::MST_Daytime,1,1,unreliable) == false)) {
-      cerr << "ERROR: Unable to initialize Daytime server!" << endl;
+      std::cerr << "ERROR: Unable to initialize Daytime server!" << std::endl;
       exit(1);
    }
 #endif
@@ -974,7 +987,7 @@ int main(int argc, char** argv)
    if((charGenServer == NULL) ||
       (charGenServer->init(localAddressArray,localAddresses,
                           CHARGEN_PORT,MultiServer::MST_CharGen,1,1,unreliable) == false)) {
-      cerr << "ERROR: Unable to initialize CharGen server!" << endl;
+      std::cerr << "ERROR: Unable to initialize CharGen server!" << std::endl;
       exit(1);
    }
 #endif
@@ -984,7 +997,7 @@ int main(int argc, char** argv)
    if((tftpServer == NULL) ||
       (tftpServer->init(localAddressArray,localAddresses,
                           TFTP_PORT,MultiServer::MST_TFTP,1,1,unreliable) == false)) {
-      cerr << "ERROR: Unable to initialize TFTP server!" << endl;
+      std::cerr << "ERROR: Unable to initialize TFTP server!" << std::endl;
       exit(1);
    }
 #endif
@@ -994,7 +1007,7 @@ int main(int argc, char** argv)
    if((testServer == NULL) ||
       (testServer->init(localAddressArray,localAddresses,
                         COMTEST_PORT,MultiServer::MST_Test,maxIncoming,maxOutgoing,unreliable)) == false) {
-      cerr << "ERROR: Unable to initialize test server!" << endl;
+      std::cerr << "ERROR: Unable to initialize test server!" << std::endl;
       exit(1);
    }
 #endif
@@ -1028,6 +1041,6 @@ int main(int argc, char** argv)
 #endif
    SocketAddress::deleteAddressList(localAddressArray);
 
-   cout << "\x1b[" << getANSIColor(0) << "mTerminated!" << endl;
+   std::cout << "\x1b[" << getANSIColor(0) << "mTerminated!" << std::endl;
    return 0;
 }
