@@ -1,38 +1,34 @@
 /*
- *  $Id: sctpterminal.cc 554 2004-11-23 10:16:14Z dreibh $
+ *  $Id$
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: SCTP Terminal
@@ -86,7 +82,6 @@ class EchoThread : public Thread
 
    Socket*  EchoSocket;
    cardinal Incoming;
-   cardinal Unreliable;
 };
 
 
@@ -113,7 +108,7 @@ void EchoThread::run()
 {
    char buffer[16384];
    for(;;) {
-      SocketMessage<CSpace(sizeof(sctp_sndrcvinfo))> message;
+      SocketMessage<sizeof(sctp_sndrcvinfo)> message;
       message.setBuffer((char*)&buffer,sizeof(buffer));
 
       // ====== Receive string from association =============================
@@ -135,7 +130,7 @@ void EchoThread::run()
                (cmsg->cmsg_type  == SCTP_SNDRCV)) {
                sctp_sndrcvinfo* info = (sctp_sndrcvinfo*)CData(cmsg);
                streamID = info->sinfo_stream;
-               protoID  = info->sinfo_ppid;
+               protoID  = ntohl(info->sinfo_ppid);
             }
             cmsg = message.getNextHeader(cmsg);
          }
@@ -150,7 +145,7 @@ void EchoThread::run()
       else {
          if(dataLength > 0) {
             if(ColorMode) {
-               cout << "\x1b[" << getANSIColor(streamID + 1) << "m";
+               std::cout << "\x1b[" << getANSIColor(streamID + 1) << "m";
             }
             if(Incoming > 1) {
                char str[128];
@@ -159,16 +154,32 @@ void EchoThread::run()
                }
                else {
                   snprintf((char*)&str,sizeof(str),"#%d/$%08x: ",
-                              streamID,protoID);
+                           streamID,protoID);
                }
-               cout << str;
+               std::cout << str;
             }
+
+            int i;
             buffer[dataLength] = 0x00;
-            cout << buffer;
-            if(ColorMode) {
-               cout << "\x1b[" << getANSIColor(0) << "m";
+            for(i = dataLength - 1;i >= 0;i--) {
+               if(buffer[i] < ' ') {
+                  buffer[i] = 0x00;
+               }
+               else {
+                  break;
+               }
             }
-            cout.flush();
+            for(   ;i >= 0;i--) {
+               if(buffer[i] < ' ') {
+                  buffer[i] = '.';
+               }
+            }
+
+            std::cout << buffer << std::endl;
+            if(ColorMode) {
+               std::cout << "\x1b[" << getANSIColor(0) << "m";
+            }
+            std::cout.flush();
          }
       }
    }
@@ -225,7 +236,11 @@ void CopyThread::run()
 
       // For some reason, cin.getline() causes problems with Ctrl-C
       // handling. Therefore, we use fgets() and feof() instead.
-      fgets((char*)&str, sizeof(str), stdin);
+      if(fgets((char*)&str, sizeof(str), stdin) == NULL) {
+         CopySocket->shutdown(2);
+         sendBreak(true);
+         return;
+      }
       if(feof(stdin)) {
          CopySocket->shutdown(2);
          sendBreak(true);
@@ -250,39 +265,42 @@ void CopyThread::run()
       if(stream >= Outgoing) {
          stream = 0;
          if(ColorMode) {
-            cerr << "\x1b[" << getANSIColor(ERROR_COLOR) << "m\x07";
+            std::cerr << "\x1b[" << getANSIColor(ERROR_COLOR) << "m\x07";
          }
-         cerr << "*** Bad stream setting! Using #0. ***" << endl;
+         std::cerr << "*** Bad stream setting! Using #0. ***" << std::endl;
          if(ColorMode) {
-            cerr << "\x1b[" << getANSIColor(0) << "m";
+            std::cerr << "\x1b[" << getANSIColor(0) << "m";
          }
-         cerr.flush();
+         std::cerr.flush();
       }
       if(update == true) {
          if(ColorMode) {
-            cout << "\x1b[" << getANSIColor(INFO_COLOR) << "m";
+            std::cout << "\x1b[" << getANSIColor(INFO_COLOR) << "m";
          }
          char str2[128];
          snprintf((char*)&str2,sizeof(str2),"Via stream #%d, protocol $%08x...",
                   stream,ppid);
-         cout << str2 << endl;
+         std::cout << str2;
          if(ColorMode) {
-            cout << "\x1b[" << getANSIColor(0) << "m";
+            std::cout << "\x1b[" << getANSIColor(0) << "m";
          }
-         cout.flush();
+         std::cout.flush();
       }
 
 
       // ====== Send message ================================================
       cardinal length = strlen(toSend);
       if(length > 0) {
-         SocketMessage<CSpace(sizeof(sctp_sndrcvinfo))> message;
+         toSend[length]   = '\n';
+         toSend[length++] = 0x00;
+
+         SocketMessage<sizeof(sctp_sndrcvinfo)> message;
          message.setBuffer(toSend,length);
          sctp_sndrcvinfo* info = (sctp_sndrcvinfo*)message.addHeader(sizeof(sctp_sndrcvinfo),IPPROTO_SCTP,SCTP_SNDRCV);
          info->sinfo_assoc_id = 0;
          info->sinfo_stream   = (unsigned short)stream;
          info->sinfo_flags    = MSG_PR_SCTP_TTL;
-         info->sinfo_ppid     = ppid;
+         info->sinfo_ppid     = htonl(ppid);
          if((Unreliable > 0) && (info->sinfo_stream <= Unreliable - 1)) {
             info->sinfo_flags      = MSG_PR_SCTP_TTL;
             info->sinfo_timetolive = 0;
@@ -294,11 +312,11 @@ void CopyThread::run()
          const ssize_t result = CopySocket->sendMsg(&message.Header,0);
          if(result < 0) {
             if(ColorMode) {
-               cerr << "\x1b[" << getANSIColor(ERROR_COLOR) << "m\x07";
+               std::cerr << "\x1b[" << getANSIColor(ERROR_COLOR) << "m\x07";
             }
-            cerr << "*** sendMsg() error #" << result << "! ***" << endl;
+            std::cerr << "*** sendMsg() error #" << result << "! ***" << std::endl;
             if(ColorMode) {
-               cerr << "\x1b[" << getANSIColor(0) << "m";
+               std::cerr << "\x1b[" << getANSIColor(0) << "m";
             }
          }
       }
@@ -319,20 +337,20 @@ int main(int argc, char** argv)
    SocketAddress** localAddressArray  = SocketAddress::newAddressList(SCTP_MAXADDRESSES);
    SocketAddress** remoteAddressArray = SocketAddress::newAddressList(SCTP_MAXADDRESSES);
    if((localAddressArray == NULL) || (remoteAddressArray == NULL)) {
-      cerr << "ERROR: Out of memory!" << endl;
+      std::cerr << "ERROR: Out of memory!" << std::endl;
       exit(1);
    }
    PrintControl       = false;
    PrintNotifications = false;
    if(!sctp_isavailable()) {
 #ifdef HAVE_KERNEL_SCTP
-      cerr << "ERROR: Kernel-based SCTP is not available!" << endl;
+      std::cerr << "ERROR: Kernel-based SCTP is not available!" << std::endl;
 #else
      if(getuid() != 0) {
-        cerr << "ERROR: You need root permissions to use the SCTP Library!" << endl;
+        std::cerr << "ERROR: You need root permissions to use the SCTP Library!" << std::endl;
      }
      else {
-        cerr << "ERROR: SCTP is not available!" << endl;
+        std::cerr << "ERROR: SCTP is not available!" << std::endl;
      }
 #endif
       exit(1);
@@ -341,9 +359,9 @@ int main(int argc, char** argv)
 
    // ====== Get arguments ==================================================
    if(argc < 2) {
-      cerr << "Usage: " << argv[0] << " "
+      std::cerr << "Usage: " << argv[0] << " "
            << "[Remote address 1] {{Remote address 2} ...} {-force-ipv4|-use-ipv6} {-local=address1} ... {-local=addressN} {-in=instreams} {-out=outstreams} {-nocolor} {-color} {-control|-nocontrol} {-notif|-nonotif}"
-           << endl;
+           << std::endl;
       exit(1);
    }
    for(unsigned int i = 1;i < (cardinal)argc;i++) {
@@ -398,13 +416,13 @@ int main(int argc, char** argv)
                SocketAddress::createSocketAddress(0,
                                                   &argv[i][7]);
             if(localAddressArray[localAddresses] == NULL) {
-               cerr << "ERROR: Argument \"" << argv[i] << "\" specifies an invalid local address!" << endl;
+               std::cerr << "ERROR: Argument \"" << argv[i] << "\" specifies an invalid local address!" << std::endl;
                exit(1);
             }
             localAddresses++;
          }
          else {
-            cerr << "ERROR: Too many local addresses!" << endl;
+            std::cerr << "ERROR: Too many local addresses!" << std::endl;
             exit(1);
          }
       }
@@ -413,13 +431,13 @@ int main(int argc, char** argv)
             remoteAddressArray[remoteAddresses] =
                SocketAddress::createSocketAddress(0, argv[i]);
             if(remoteAddressArray[remoteAddresses] == NULL) {
-               cerr << "ERROR: Argument \"" << argv[i] << "\" is an invalid remote address!" << endl;
+               std::cerr << "ERROR: Argument \"" << argv[i] << "\" is an invalid remote address!" << std::endl;
                exit(1);
             }
             remoteAddresses++;
          }
          else {
-            cerr << "ERROR: Too many remote addresses!" << endl;
+            std::cerr << "ERROR: Too many remote addresses!" << std::endl;
             exit(1);
          }
       }
@@ -428,32 +446,33 @@ int main(int argc, char** argv)
       InternetAddress::UseIPv6 = false;
    }
    if(remoteAddresses < 1) {
-      cerr << "ERROR: No remote addresses given!" << endl;
+      std::cerr << "ERROR: No remote addresses given!" << std::endl;
       exit(1);
    }
    if(remoteAddressArray[0]->getPort() == 0) {
-      cerr << "ERROR: No remote port number is given with first remote address!" << endl;
+      std::cerr << "ERROR: No remote port number is given with first remote address!" << std::endl;
       exit(1);
    }
    if(localAddresses < 1) {
 /*
       localAddressArray[0] = new InternetAddress(0);
       if(localAddressArray[0] == NULL) {
-         cerr << "ERROR: Out of memory!" << endl;
+         std::cerr << "ERROR: Out of memory!" << std::endl;
          exit(1);
       }
       localAddresses = 1;
 */
+      SocketAddress::deleteAddressList(localAddressArray);
       if(!Socket::getLocalAddressList(
             localAddressArray,
             localAddresses,
             Socket::GLAF_HideBroadcast|Socket::GLAF_HideMulticast|Socket::GLAF_HideAnycast)) {
-         cerr << "ERROR: Cannot obtain local addresses!" << endl;
+         std::cerr << "ERROR: Cannot obtain local addresses!" << std::endl;
          exit(1);
       }
       if(localAddresses < 1) {
-         cerr << "ERROR: No valid local addresses have been found?!" << endl
-              << "       Check your network interface configuration!" << endl;
+         std::cerr << "ERROR: No valid local addresses have been found?!" << std::endl
+                   << "       Check your network interface configuration!" << std::endl;
          exit(1);
       }
    }
@@ -463,29 +482,28 @@ int main(int argc, char** argv)
    for(cardinal i = 1;i < remoteAddresses;i++) {
       remoteAddressArray[i]->setPort(remoteAddressArray[0]->getPort());
    }
-   unreliable = min(outstreams,unreliable);
+   unreliable = std::min(outstreams,unreliable);
 
 
    // ====== Print information ==============================================
-   cout << "SCTP Terminal - Copyright (C) 2001-2003 Thomas Dreibholz" << endl;
-   cout << "--------------------------------------------------------" << endl;
-   cout << "Version:               " << __DATE__ << ", " << __TIME__ << endl;
+   std::cout << "SCTP Terminal - Copyright (C) 2001-2026 Thomas Dreibholz" << std::endl;
+   std::cout << "--------------------------------------------------------" << std::endl;
    localAddressArray[0]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
-   cout << "Local Addresses:       " << *(localAddressArray[0]) << endl;
+   std::cout << "Local Addresses:       " << *(localAddressArray[0]) << std::endl;
    for(cardinal i = 1;i < localAddresses;i++) {
       localAddressArray[i]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
-      cout << "                       " << *(localAddressArray[i]) << endl;
+      std::cout << "                       " << *(localAddressArray[i]) << std::endl;
    }
-   cout << "Remote Addresses:      " << *(remoteAddressArray[0]) << endl;
+   std::cout << "Remote Addresses:      " << *(remoteAddressArray[0]) << std::endl;
    remoteAddressArray[0]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
    for(cardinal i = 1;i < remoteAddresses;i++) {
       remoteAddressArray[i]->setPrintFormat(SocketAddress::PF_Address|SocketAddress::PF_Legacy|SocketAddress::PF_HidePort);
-      cout << "                       " << *(remoteAddressArray[i]) << endl;
+      std::cout << "                       " << *(remoteAddressArray[i]) << std::endl;
    }
-   cout << "Outgoing Streams:      " << outstreams     << endl;
-   cout << "Max. Incoming Streams: " << instreams      << endl;
-   cout << "Unreliable:            " << unreliable     << endl;
-   cout << endl << endl;
+   std::cout << "Outgoing Streams:      " << outstreams << std::endl;
+   std::cout << "Max. Incoming Streams: " << instreams  << std::endl;
+   std::cout << "Unreliable:            " << unreliable << std::endl;
+   std::cout << std::endl << std::endl;
 
 
    // ====== Create socket and connect ======================================
@@ -493,7 +511,7 @@ int main(int argc, char** argv)
    if(clientSocket.bindx((const SocketAddress**)localAddressArray,
                          localAddresses,
                          SCTP_BINDX_ADD_ADDR) == false) {
-      cerr << "ERROR: Unable to bind socket!" << endl;
+      std::cerr << "ERROR: Unable to bind socket!" << std::endl;
       exit(1);
    }
 
@@ -503,26 +521,26 @@ int main(int argc, char** argv)
    init.sinit_max_attempts   = 0;
    init.sinit_max_init_timeo = 60;
    if(clientSocket.setSocketOption(IPPROTO_SCTP,SCTP_INITMSG,(char*)&init,sizeof(init)) < 0) {
-      cerr << "ERROR: Unable to set SCTP_INITMSG parameters!" << endl;
+      std::cerr << "ERROR: Unable to set SCTP_INITMSG parameters!" << std::endl;
       exit(1);
    }
 
    sctp_event_subscribe events;
    memset((char*)&events,1,sizeof(events));
    if(clientSocket.setSocketOption(IPPROTO_SCTP,SCTP_EVENTS,&events,sizeof(events)) < 0) {
-      cerr << "ERROR: SCTP_EVENTS failed!" << endl;
+      std::cerr << "ERROR: SCTP_EVENTS failed!" << std::endl;
       exit(1);
    }
 
-   cout << "Connecting... ";
-   cout.flush();
+   std::cout << "Connecting... ";
+   std::cout.flush();
    if(clientSocket.connectx((const SocketAddress**)remoteAddressArray,
                             remoteAddresses) == false) {
-      cout << "failed!" << endl;
-      cerr << "ERROR: Unable to connect to remote address(es)!" << endl;
+      std::cout << "failed!" << std::endl;
+      std::cerr << "ERROR: Unable to connect to remote address(es)!" << std::endl;
       exit(1);
    }
-   cout << "done. Use Ctrl-D to end transmission." << endl << endl;
+   std::cout << "done. Use Ctrl-D to end transmission." << std::endl << std::endl;
 
 
    // ====== Start threads ==================================================
@@ -542,6 +560,6 @@ int main(int argc, char** argv)
    echo.stop();
    SocketAddress::deleteAddressList(localAddressArray);
    SocketAddress::deleteAddressList(remoteAddressArray);
-   cout << "\x1b[" << getANSIColor(0) << "mTerminated!" << endl;
+   std::cout << "\x1b[" << getANSIColor(0) << "mTerminated!" << std::endl;
    return 0;
 }
