@@ -1,38 +1,34 @@
 /*
- *  $Id: multitimerthread.h 183 2003-08-19 19:28:34Z tuexen $
+ *  $Id$
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 2005-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Multi Timer Thread Implementation
@@ -45,6 +41,7 @@
 
 
 #include "tdsystem.h"
+#include "tools.h"
 #include "thread.h"
 
 
@@ -58,7 +55,7 @@
   * (Default is on at a maximum of 10 calls).
   *
   * @short   Multi Timer Thread
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   * @see Thread
   */
@@ -103,7 +100,7 @@ template<const cardinal Timers> class MultiTimerThread : public Thread
      * @param timer Timer number.
      * @return Interval in microseconds.
      */
-   inline card64 getInterval(const cardinal timer) const;
+   inline card64 getInterval(const cardinal timer);
 
    /**
      * Set timer interval. Note, that the first timerEvent() call will
@@ -159,7 +156,7 @@ template<const cardinal Timers> class MultiTimerThread : public Thread
      *
      * @see setTimerCorrection
      */
-   inline cardinal getTimerCorrection(const cardinal timer) const;
+   inline cardinal getTimerCorrection(const cardinal timer);
 
    /**
      * Set correction of inaccurate system timer to given value.
@@ -233,6 +230,7 @@ template<const cardinal Timers> class MultiTimerThread : public Thread
    // ====== Private data ===================================================
    private:
    void run();
+   inline bool isShuttingDown();
 
    struct TimerParameters {
       card64   Interval;
@@ -257,7 +255,6 @@ typedef MultiTimerThread<1> SingleTimerThread;
 
 
 #include "multitimerthread.icc"
-#include "multitimerthread.cc"
 
 
 #endif
