@@ -1,38 +1,34 @@
 /*
- *  $Id: tdsocket.cc 509 2004-11-11 21:35:41Z dreibh $
+ *  $Id$
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 2005-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Socket Implementation
@@ -125,10 +121,10 @@ void Socket::packSocketAddressArray(const sockaddr_storage* addrArray,
             a = (sockaddr*)((long)a + (long)sizeof(sockaddr_in6));
           break;
          default:
-            cerr << "ERROR: pack_sockaddr_storage() - Unknown address type #" << ((sockaddr*)&addrArray[i])->sa_family << "!" << endl;
-            cerr << "IMPORTANT NOTE:" << endl
-                 << "The standardizers have changed the socket API; the sockaddr_storage array has been replaced by a variable-sized sockaddr_in/in6 blocks. Do not blame us for this change, send your complaints to the standardizers at sctp-impl@external.cisco.com!" << endl;
-            exit(1);
+            std::cerr << "ERROR: pack_sockaddr_storage() - Unknown address type #" << ((sockaddr*)&addrArray[i])->sa_family << "!" << std::endl;
+            std::cerr << "IMPORTANT NOTE:" << std::endl
+                      << "The standardizers have changed the socket API; the sockaddr_storage array has been replaced by a variable-sized sockaddr_in/in6 blocks. Do not blame us for this change, send your complaints to the standardizers at sctp-impl@external.cisco.com!" << std::endl;
+            ::abort();
           break;
       }
    }
@@ -138,17 +134,15 @@ void Socket::packSocketAddressArray(const sockaddr_storage* addrArray,
 // ###### Initialize socket #################################################
 void Socket::init()
 {
-   CommunicationDomain = UndefinedSocketCommunicationDomain;
-   Type                = UndefinedSocketType;
-   Protocol            = UndefinedSocketProtocol;
-   Destination         = NULL;
-   Backlog             = 0;
-   SendFlow            = 0;
-   ReceivedFlow        = 0;
-   BytesSent           = 0;
-   BytesReceived       = 0;
-   SocketDescriptor    = -1;
-   LastError           = 0;
+   SocketDescriptor = -1;
+   Family           = UndefinedSocketFamily;
+   Type             = UndefinedSocketType;
+   Protocol         = UndefinedSocketProtocol;
+   Destination      = NULL;
+   SendFlow         = 0;
+   ReceivedFlow     = 0;
+   LastError        = 0;
+   Backlog          = 0;
 }
 
 
@@ -158,40 +152,40 @@ bool Socket::create(const integer communicationDomain,
                     const integer socketProtocol)
 {
    close();
-   CommunicationDomain = communicationDomain;
-   Type                = socketType;
-   Protocol            = socketProtocol;
-   if(CommunicationDomain == IP) {
+   Family   = communicationDomain;
+   Type     = socketType;
+   Protocol = socketProtocol;
+   if(Family == IP) {
       if(InternetAddress::UseIPv6) {
-         CommunicationDomain = IPv6;
+         Family = IPv6;
       }
       else
-         CommunicationDomain = IPv4;
+         Family = IPv4;
    }
 
    // ====== Create socket ==================================================
-   int result = ext_socket(CommunicationDomain,socketType,socketProtocol);
-   if(result < 0) {
+   SocketDescriptor = ext_socket(Family,socketType,socketProtocol);
+   if(SocketDescriptor < 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::Socket() - Unable to create socket!" << endl;
+      std::cerr << "WARNING: Socket::Socket() - Unable to create socket!" << std::endl;
 #endif
       return(false);
    }
-   SocketDescriptor = result;
 
    // ====== Set options ====================================================
+#if (SYSTEM == OS_Linux)
+   const int yes = 1;
    // Send and receive IPv6 flow labels.
-#if (SYSTEM == OS_Linux)
-   int opt1 = 1;
-   setsockopt(SocketDescriptor,SOL_IPV6,IPV6_FLOWINFO,&opt1,sizeof(opt1));
-   setsockopt(SocketDescriptor,SOL_IPV6,IPV6_FLOWINFO_SEND,&opt1,sizeof(opt1));
-#endif
-
+   ext_setsockopt(SocketDescriptor,SOL_IPV6,IPV6_FLOWINFO,&yes,sizeof(yes));
+   ext_setsockopt(SocketDescriptor,SOL_IPV6,IPV6_FLOWINFO_SEND,&yes,sizeof(yes));
    // Receive IPv4 TOS field.
-#if (SYSTEM == OS_Linux)
-   bool opt2 = true;
-   setsockopt(SocketDescriptor,SOL_IP,IP_RECVTOS,&opt2,sizeof(opt2));
+   ext_setsockopt(SocketDescriptor,SOL_IP,IP_RECVTOS,&yes,sizeof(yes));
 #endif
+   if(Family == IPv6) {
+      const int no = 0;
+      ext_setsockopt(SocketDescriptor,IPPROTO_IPV6,IPV6_V6ONLY,&no,sizeof(no));
+   }
+
    return(true);
 }
 
@@ -228,7 +222,7 @@ bool Socket::bind(const SocketAddress& address)
    sockaddr_in6* socketAddress       = (sockaddr_in6*)&socketAddressBuffer;
    socklen_t     socketAddressLength =
       address.getSystemAddress((sockaddr*)socketAddress,SocketAddress::MaxSockLen,
-                               CommunicationDomain);
+                               Family);
    if(socketAddressLength == 0) {
       LastError = ENAMETOOLONG;
       return(false);
@@ -300,8 +294,8 @@ bool Socket::bindx(const SocketAddress** addressArray,
                                            sizeof(sockaddr_storage),
                                            AF_UNSPEC) == false) {
 #ifndef DISABLE_WARNINGS
-         cerr << "ERROR: Socket::bindx() - Unable to get system socket address for "
-              << *(addressArray[i]) << "!" << endl;
+         std::cerr << "ERROR: Socket::bindx() - Unable to get system socket address for "
+                   << *(addressArray[i]) << "!" << std::endl;
 #endif
          return(false);
       }
@@ -571,7 +565,7 @@ Socket* Socket::accept(SocketAddress** address)
    Socket* accepted = new Socket;
    if(accepted != NULL) {
       accepted->SocketDescriptor    = result;
-      accepted->CommunicationDomain = CommunicationDomain;
+      accepted->Family = Family;
       accepted->Type                = Type;
       accepted->Protocol            = Protocol;
       if(address != NULL) {
@@ -582,7 +576,7 @@ Socket* Socket::accept(SocketAddress** address)
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::accept() - Out of memory!" << endl;
+      std::cerr << "WARNING: Socket::accept() - Out of memory!" << std::endl;
 #endif
       ext_close(result);
    }
@@ -598,7 +592,7 @@ bool Socket::connect(const SocketAddress& address, const card8 trafficClass)
    sockaddr_in6* socketAddress       = (sockaddr_in6*)&socketAddressBuffer;
    socklen_t     socketAddressLength =
       address.getSystemAddress((sockaddr*)socketAddress,SocketAddress::MaxSockLen,
-                               CommunicationDomain);
+                               Family);
    if(socketAddressLength == 0) {
       return(false);
    }
@@ -628,7 +622,7 @@ bool Socket::connect(const SocketAddress& address, const card8 trafficClass)
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::connect() - Out of memory!" << endl;
+      std::cerr << "WARNING: Socket::connect() - Out of memory!" << std::endl;
 #endif
       return(false);
    }
@@ -652,22 +646,22 @@ bool Socket::connectx(const SocketAddress** addressArray,
 {
    // ====== Get address ====================================================
    sockaddr_storage socketAddressArray[addresses];
-   socklen_t        socketAddressLength[addresses];
+   // socklen_t        socketAddressLength[addresses];
 
    for(cardinal i = 0;i < addresses;i++) {
-      socketAddressLength[i] =
+      // socketAddressLength[i] =
          addressArray[i]->getSystemAddress((sockaddr*)&socketAddressArray[i],
                                            sizeof(socketAddressArray[addresses]),
-                                           CommunicationDomain);
+                                           Family);
    }
    Destination = NULL;
 
    // ====== Connect ========================================================
    sockaddr_storage packedSocketAddressArray[addresses];
    packSocketAddressArray(socketAddressArray, addresses, (sockaddr*)&packedSocketAddressArray);
-   int result = ext_connectx(SocketDescriptor,
-                             (sockaddr*)&packedSocketAddressArray,
-                             addresses);
+   int result = sctp_connectx(SocketDescriptor,
+                              (sockaddr*)&packedSocketAddressArray,
+                              addresses, NULL);
    if(result != 0) {
       LastError = errno;
       if(LastError != EINPROGRESS) {
@@ -681,17 +675,21 @@ bool Socket::connectx(const SocketAddress** addressArray,
 
 // ###### Receive message ###################################################
 ssize_t Socket::receiveMsg(struct msghdr* msg,
-                           const cardinal flags,
+                           const integer  flags,
                            const bool     internalCall)
 {
-   int cc = ext_recvmsg2(SocketDescriptor,msg,flags,(internalCall == true) ? 0 : 1);
+#ifdef SOCKETAPI_MAJOR_VERSION
+   const int cc = ext_recvmsg2(SocketDescriptor,msg,flags,(internalCall == true) ? 0 : 1);
+#else
+   const int cc = ext_recvmsg(SocketDescriptor,msg,flags);
+#endif
    if(cc < 0) {
       LastError = errno;
       return(-LastError);
    }
 
    ReceivedFlow = 0;
-   for(cmsghdr* c = CFirst(msg);c;c = CNext(msg,c)) {
+   for(cmsghdr* c = CFirstHeader(msg);c;c = CNextHeader(msg,c)) {
       switch(c->cmsg_level) {
 #if (SYSTEM == OS_Linux)
          case SOL_IPV6:
@@ -722,7 +720,7 @@ ssize_t Socket::receiveMsg(struct msghdr* msg,
 ssize_t Socket::recvFrom(int              fd,
                          void*            buf,
                          const size_t     len,
-                         const integer    flags,
+                         integer&         flags,
                          struct sockaddr* addr,
                          socklen_t*       addrlen)
 {
@@ -738,13 +736,15 @@ ssize_t Socket::recvFrom(int              fd,
       *addrlen,
       &iov, 1,
       cbuf, sizeof(cbuf),
-      0
+      flags
    };
 
    cc = receiveMsg(&msg,flags,true);
    if(cc < 0) {
       return(cc);
    }
+
+   flags = msg.msg_flags;
    *addrlen = msg.msg_namelen;
    return(cc);
 }
@@ -754,29 +754,28 @@ ssize_t Socket::recvFrom(int              fd,
 ssize_t Socket::receiveFrom(void*          buffer,
                             const size_t   length,
                             SocketAddress& sender,
-                            const cardinal flags)
+                            integer&       flags)
 {
    char      socketAddressBuffer[SocketAddress::MaxSockLen];
    socklen_t socketAddressLength = SocketAddress::MaxSockLen;
 
-   ssize_t result = recvFrom(SocketDescriptor, buffer,
-                             length, flags,
-                             (sockaddr*)&socketAddressBuffer,
-                             &socketAddressLength);
+   const ssize_t result = recvFrom(SocketDescriptor, buffer,
+                                   length, flags,
+                                   (sockaddr*)&socketAddressBuffer,
+                                   &socketAddressLength);
    if(result > 0) {
       sender.setSystemAddress((sockaddr*)&socketAddressBuffer,
                               socketAddressLength);
-      BytesReceived += (card64)result;
    }
    return(result);
 }
 
 
 // ###### Send data #########################################################
-ssize_t Socket::send(const void*    buffer,
-                     const size_t   length,
-                     const cardinal flags,
-                     const card8    trafficClass)
+ssize_t Socket::send(const void*   buffer,
+                     const size_t  length,
+                     const integer flags,
+                     const card8   trafficClass)
 {
    // ====== Check, if traffic class has to be set ==========================
    if((trafficClass != 0) && (Destination != NULL)) {
@@ -790,12 +789,10 @@ ssize_t Socket::send(const void*    buffer,
                                            & IPV6_FLOWINFO_FLOWLABEL) |
                                            ((card32)trafficClass << 20));
          ssize_t result = ext_sendto(SocketDescriptor, buffer, length, flags,
-                                       (sockaddr*)&newAddress, sizeof(sockaddr_in6));
-         if(result > 0) {
-            BytesSent += (card64)result;
-         }
-         else {
+                                     (sockaddr*)&newAddress, sizeof(sockaddr_in6));
+         if(result < 0) {
             LastError = errno;
+            result    = -LastError;
          }
          return(result);
       }
@@ -806,10 +803,7 @@ ssize_t Socket::send(const void*    buffer,
          setTypeOfService(trafficClass);
          ssize_t result = ext_send(SocketDescriptor,buffer,length,flags);
          setTypeOfService(SendFlow >> 20);
-         if(result > 0) {
-            BytesSent += (card64)result;
-         }
-         else {
+         if(result < 0) {
             LastError = errno;
             result    = -LastError;
          }
@@ -819,10 +813,7 @@ ssize_t Socket::send(const void*    buffer,
 
    // ====== Do simple send() without setting traffic class =================
    ssize_t result = ext_send(SocketDescriptor,buffer,length,flags);
-   if(result > 0) {
-      BytesSent += (card64)result;
-   }
-   else {
+   if(result < 0) {
       LastError = errno;
       result    = -LastError;
    }
@@ -833,7 +824,7 @@ ssize_t Socket::send(const void*    buffer,
 // ###### Send data to receiver address #####################################
 ssize_t Socket::sendTo(const void*          buffer,
                        const size_t         length,
-                       const cardinal       flags,
+                       const integer        flags,
                        const SocketAddress& receiver,
                        const card8          trafficClass)
 {
@@ -842,7 +833,7 @@ ssize_t Socket::sendTo(const void*          buffer,
    sockaddr_in6* socketAddress       = (sockaddr_in6*)&socketAddressBuffer;
    socklen_t     socketAddressLength =
       receiver.getSystemAddress((sockaddr*)socketAddress,SocketAddress::MaxSockLen,
-                               CommunicationDomain);
+                               Family);
    if(socketAddressLength == 0) {
       return(-1);
    }
@@ -859,11 +850,9 @@ ssize_t Socket::sendTo(const void*          buffer,
                                            ((card32)trafficClass << 20));
          ssize_t result = ext_sendto(SocketDescriptor,buffer,length,flags,
                                       (sockaddr*)&newAddress,sizeof(sockaddr_in6));
-         if(result > 0) {
-            BytesSent += (card64)result;
-         }
-         else {
+         if(result < 0) {
             LastError = errno;
+            result    = -LastError;
          }
          return(result);
       }
@@ -875,11 +864,9 @@ ssize_t Socket::sendTo(const void*          buffer,
          ssize_t result = ext_sendto(SocketDescriptor,buffer,length,flags,
                                        (sockaddr*)socketAddress,socketAddressLength);
          setTypeOfService(SendFlow >> 20);
-         if(result > 0) {
-            BytesSent += (card64)result;
-         }
-         else {
+         if(result < 0) {
             LastError = errno;
+            result    = -LastError;
          }
          return(result);
       }
@@ -888,11 +875,9 @@ ssize_t Socket::sendTo(const void*          buffer,
    // ====== Do simple send() without setting traffic class =================
    ssize_t result = ext_sendto(SocketDescriptor,buffer,length,flags,
                                  (sockaddr*)socketAddress,socketAddressLength);
-   if(result > 0) {
-      BytesSent += (card64)result;
-   }
-   else {
+   if(result < 0) {
       LastError = errno;
+      result    = -LastError;
    }
    return(result);
 }
@@ -900,20 +885,19 @@ ssize_t Socket::sendTo(const void*          buffer,
 
 // ###### Send message ######################################################
 ssize_t Socket::sendMsg(const struct msghdr* msg,
-                        const cardinal       flags,
+                        const integer        flags,
                         const card8          trafficClass)
 {
    if(trafficClass != 0x00) {
       setTypeOfService(trafficClass);
    }
+
    ssize_t result = ext_sendmsg(SocketDescriptor,msg,(int)flags);
-   if(result >= 0) {
-      BytesSent += (card64)result;
-   }
-   else {
+   if(result < 0) {
       LastError = errno;
       result    = -LastError;
    }
+
    if(trafficClass != 0x00) {
       setTypeOfService(SendFlow >> 20);
    }
@@ -958,7 +942,7 @@ InternetFlow Socket::allocFlow(const InternetAddress& address,
                                     sizeof(in6_flowlabel_req));
    if(result != 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: InternetFlow::allocFlow() - Unable to get flow label!" << endl;
+      std::cerr << "WARNING: InternetFlow::allocFlow() - Unable to get flow label!" << std::endl;
 #endif
       InternetFlow flow;
       return(flow);
@@ -993,7 +977,7 @@ void Socket::freeFlow(InternetFlow& flow)
                                     sizeof(in6_flowlabel_req));
    if(result != 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: InternetFlow::freeFlow() - Unable to free flow label!" << endl;
+      std::cerr << "WARNING: InternetFlow::freeFlow() - Unable to free flow label!" << std::endl;
 #endif
    }
 #endif
@@ -1024,7 +1008,7 @@ bool Socket::renewFlow(InternetFlow&  flow,
                                     sizeof(in6_flowlabel_req));
    if(result != 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::renew() - Unable to renew flow label!" << endl;
+      std::cerr << "WARNING: Socket::renew() - Unable to renew flow label!" << std::endl;
 #endif
       return(false);
    }
@@ -1057,7 +1041,7 @@ bool Socket::renewFlow(const cardinal expires,
                                     sizeof(in6_flowlabel_req));
    if(result != 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::renew() - Unable to renew flow label!" << endl;
+      std::cerr << "WARNING: Socket::renew() - Unable to renew flow label!" << std::endl;
 #endif
       return(false);
    }
@@ -1096,7 +1080,7 @@ bool Socket::getPeerAddress(SocketAddress& address) const
 // ###### Get blocking mode #################################################
 bool Socket::getBlockingMode()
 {
-   const long result = fcntl(F_GETFL,0);
+   const long result = fcntl(F_GETFL,(long)0);
    return(!(result & O_NONBLOCK));
 }
 
@@ -1104,7 +1088,7 @@ bool Socket::getBlockingMode()
 // ###### Set blocking mode #################################################
 bool Socket::setBlockingMode(const bool on)
 {
-   long flags = fcntl(F_GETFL,0);
+   long flags = fcntl(F_GETFL,(long)0);
    if(flags != -1) {
       flags = on ? (flags & ~O_NONBLOCK) : (flags | O_NONBLOCK);
       return(fcntl(F_SETFL,flags) == 0);
@@ -1193,11 +1177,11 @@ bool Socket::multicastMembership(const SocketAddress& address,
                                  const char*          interface,
                                  const bool           add)
 {
-   if(CommunicationDomain == AF_INET) {
+   if(Family == AF_INET) {
       sockaddr_in addr;
       if(address.getSystemAddress((sockaddr*)&addr,sizeof(addr),AF_INET) == 0) {
 #ifndef DISABLE_WARNINGS
-         cerr << "ERROR: Socket::multicastMembership() - Bad address type for IPv4 socket!" << endl;
+         std::cerr << "ERROR: Socket::multicastMembership() - Bad address type for IPv4 socket!" << std::endl;
 #endif
       }
       else {
@@ -1208,7 +1192,7 @@ bool Socket::multicastMembership(const SocketAddress& address,
             strcpy(ifr.ifr_name,interface);
             if(ioctl(SIOCGIFADDR,&ifr) != 0) {
 #ifndef DISABLE_WARNINGS
-               cerr << "ERROR: Socket::multicastMembership() - Unable to get interface address!" << endl;
+               std::cerr << "ERROR: Socket::multicastMembership() - Unable to get interface address!" << std::endl;
 #endif
                return(false);
             }
@@ -1222,11 +1206,11 @@ bool Socket::multicastMembership(const SocketAddress& address,
                                 &mreq,sizeof(mreq)));
       }
    }
-   else if(CommunicationDomain == AF_INET6) {
+   else if(Family == AF_INET6) {
       sockaddr_in6 addr;
       if(address.getSystemAddress((sockaddr*)&addr,sizeof(addr),AF_INET6) == 0) {
 #ifndef DISABLE_WARNINGS
-         cerr << "ERROR: Socket::multicastMembership() - Bad address type for IPv6 socket!" << endl;
+         std::cerr << "ERROR: Socket::multicastMembership() - Bad address type for IPv6 socket!" << std::endl;
 #endif
       }
       else {
@@ -1245,7 +1229,7 @@ bool Socket::multicastMembership(const SocketAddress& address,
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::multicastMembership() - Multicast is not supported for this socket type!" << endl;
+      std::cerr << "ERROR: Socket::multicastMembership() - Multicast is not supported for this socket type!" << std::endl;
 #endif
    }
    return(false);
@@ -1255,14 +1239,14 @@ bool Socket::multicastMembership(const SocketAddress& address,
 // ###### Get multicast loop mode ###########################################
 bool Socket::getMulticastLoop()
 {
-   if(CommunicationDomain == AF_INET) {
+   if(Family == AF_INET) {
       u_char    loop;
       socklen_t size = sizeof(loop);
       if(getSocketOption(IPPROTO_IP,IP_MULTICAST_LOOP,&loop,&size) == 0) {
          return(loop);
       }
    }
-   else if(CommunicationDomain == AF_INET6) {
+   else if(Family == AF_INET6) {
       int       loop;
       socklen_t size = sizeof(loop);
       if(getSocketOption(IPPROTO_IPV6,IPV6_MULTICAST_LOOP,&loop,&size) == 0) {
@@ -1271,7 +1255,7 @@ bool Socket::getMulticastLoop()
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::getMulticastLoop() - Multicast is not supported for this socket type!" << endl;
+      std::cerr << "ERROR: Socket::getMulticastLoop() - Multicast is not supported for this socket type!" << std::endl;
 #endif
    }
    return(false);
@@ -1281,17 +1265,17 @@ bool Socket::getMulticastLoop()
 // ###### Set multicast loop mode ###########################################
 bool Socket::setMulticastLoop(const bool on)
 {
-   if(CommunicationDomain == AF_INET) {
+   if(Family == AF_INET) {
       const u_char value = (on ? 1 : 0);
       return(setSocketOption(IPPROTO_IP,IP_MULTICAST_LOOP,&value,sizeof(value)) == 0);
    }
-   else if(CommunicationDomain == AF_INET6) {
+   else if(Family == AF_INET6) {
       const int value = (on ? 1 : 0);
       return(setSocketOption(IPPROTO_IPV6,IPV6_MULTICAST_LOOP,&value,sizeof(value)) == 0);
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::setMulticastLoop() - Multicast is not supported for this socket type!" << endl;
+      std::cerr << "ERROR: Socket::setMulticastLoop() - Multicast is not supported for this socket type!" << std::endl;
 #endif
    }
    return(false);
@@ -1301,14 +1285,14 @@ bool Socket::setMulticastLoop(const bool on)
 // ###### Get multicast TTL #################################################
 card8 Socket::getMulticastTTL()
 {
-   if(CommunicationDomain == AF_INET) {
+   if(Family == AF_INET) {
       u_char    ttl;
       socklen_t size = sizeof(ttl);
       if(getSocketOption(IPPROTO_IP,IP_MULTICAST_TTL,&ttl,&size) == 0) {
          return(ttl);
       }
    }
-   else if(CommunicationDomain == AF_INET6) {
+   else if(Family == AF_INET6) {
       int       ttl;
       socklen_t size = sizeof(ttl);
       if(getSocketOption(IPPROTO_IPV6,IPV6_MULTICAST_HOPS,&ttl,&size) == 0) {
@@ -1317,7 +1301,7 @@ card8 Socket::getMulticastTTL()
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::getMulticastTTL() - Multicast is not supported for this socket type!" << endl;
+      std::cerr << "ERROR: Socket::getMulticastTTL() - Multicast is not supported for this socket type!" << std::endl;
 #endif
    }
    return(0);
@@ -1327,16 +1311,16 @@ card8 Socket::getMulticastTTL()
 // ###### Set multicast TTL #################################################
 bool Socket::setMulticastTTL(const card8 ttl)
 {
-   if(CommunicationDomain == AF_INET) {
+   if(Family == AF_INET) {
       return(setSocketOption(IPPROTO_IP,IP_MULTICAST_TTL,&ttl,sizeof(ttl)) == 0);
    }
-   else if(CommunicationDomain == AF_INET6) {
+   else if(Family == AF_INET6) {
       const int value = (int)ttl;
       return(setSocketOption(IPPROTO_IPV6,IPV6_MULTICAST_HOPS,&value,sizeof(value)) == 0);
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::setMulticastTTL() - Multicast is not supported for this socket type!" << endl;
+      std::cerr << "ERROR: Socket::setMulticastTTL() - Multicast is not supported for this socket type!" << std::endl;
 #endif
    }
    return(false);
@@ -1353,7 +1337,7 @@ bool Socket::setTypeOfService(const card8 trafficClass)
       char str[32];
       snprintf((char*)&str,sizeof(str),"$%02x!",trafficClass);
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: Socket::setTypeOfService() - Unable to set TOS to " << str << endl;
+      std::cerr << "WARNING: Socket::setTypeOfService() - Unable to set TOS to " << str << std::endl;
 #endif
       return(false);
    }
@@ -1406,7 +1390,7 @@ bool Socket::getLocalAddressList(SocketAddress**& addressList,
    }
    if(::ioctl(fd,SIOCGIFCONF,(char*)&config) == -1) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: Socket::getLocalAddressList() - SIOCGIFCONF failed!" << endl;
+      std::cerr << "ERROR: Socket::getLocalAddressList() - SIOCGIFCONF failed!" << std::endl;
 #endif
       ::close(fd);
       return(false);
@@ -1537,7 +1521,7 @@ bool Socket::getLocalAddressList(SocketAddress**& addressList,
       memcpy(local.ifr_name,ifrequest->ifr_name,IFNAMSIZ);
       if(::ioctl(fd,SIOCGIFFLAGS,(char*)&local) < 0) {
 #ifndef DISABLE_WARNINGS
-         cerr << "ERROR: Socket::getLocalAddressList() - SIOCGIFFLAGS failed!" << endl;
+         std::cerr << "ERROR: Socket::getLocalAddressList() - SIOCGIFFLAGS failed!" << std::endl;
 #endif
          continue;
       }
