@@ -1,38 +1,34 @@
 /*
- *  $Id: tdsocket.h 183 2003-08-19 19:28:34Z tuexen $
+ *  $Id$
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 2005-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Socket Implementation
@@ -75,20 +71,20 @@ const cardinal IPv6HeaderSize = 40;
   * when supported by the system.
   *
   * @short   Socket
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
-  */            
+  */
 class Socket
 {
    // ====== Definitions ====================================================
    public:
-   enum SocketCommunicationDomain {
-      UndefinedSocketCommunicationDomain = -1,
-      IP                                 = 255,
-      IPv4                               = AF_INET,  // Do not use IPv4/IPv6,
-      IPv6                               = AF_INET6, // use IP instead!
-      Unix                               = AF_UNIX
-   }; 
+   enum SocketFamily {
+      UndefinedSocketFamily = -1,
+      IP                    = 255,
+      IPv4                  = AF_INET,  // Do not use IPv4/IPv6,
+      IPv6                  = AF_INET6, // use IP instead!
+      Unix                  = AF_UNIX
+   };
    enum SocketType {
       UndefinedSocketType = -1,
       UDP                 = SOCK_DGRAM,
@@ -127,17 +123,17 @@ class Socket
 
    /**
      * Constructor for a new socket. For automatic usage of IPv6 when available,
-     * set communication domain to IP. Use IPv4/IPv6 only if a special
+     * set communication family to IP. Use IPv4/IPv6 only if a special
      * protocol version is necessary!
      * The creation success can be checked using ready() method.
      *
-     * @param communicationDomain Communication domain (e.g. IP).
+     * @param family Communication family (e.g. IP).
      * @param socketType Socket type (e.g. TCP, UDP).
      * @param socketProtocol Socket protocol (e.g. Default).
      *
      * @see ready
      */
-   Socket(const integer communicationDomain,
+   Socket(const integer family,
           const integer socketType,
           const integer socketProtocol = Default);
 
@@ -150,17 +146,17 @@ class Socket
    // ====== Create/close socket ============================================
    /**
      * Close existing socket and create new socket. For automatic usage of
-     * IPv6 when available, set communication domain to IP.
+     * IPv6 when available, set communication family to IP.
      * Use IPv4/IPv6 only if a special protocol version is necessary!
      *
-     * @param communicationDomain Communication domain (e.g. IP).
+     * @param socketFamily Communication family (e.g. IP).
      * @param socketType Socket type (e.g. TCP, UDP).
      * @param socketProtocol Socket protocol (e.g. Default).
      * @return true, if creation was sucessful; false otherwise.
      */
-   bool create(const integer communicationDomain = IP,
-               const integer socketType          = TCP,
-               const integer socketProtocol      = Default);
+   bool create(const integer socketFamily   = IP,
+               const integer socketType     = TCP,
+               const integer socketProtocol = Default);
 
    /**
      * Close socket.
@@ -179,6 +175,13 @@ class Socket
 
 
    // ====== Socket properties ==============================================
+   /**
+     * Get socket's family.
+     *
+     * @return Socket family.
+     */
+   inline integer getFamily() const;
+
    /**
      * Get socket's type.
      *
@@ -186,13 +189,6 @@ class Socket
      */
    inline integer getType() const;
 
-   /**
-     * Get socket's communication domain.
-     *
-     * @return Socket communication domain.
-     */
-   inline integer getCommunicationDomain() const;
-
    /**
      * Get socket's protocol.
      *
@@ -201,32 +197,6 @@ class Socket
    inline integer getProtocol() const;
 
 
-   // ====== Statistics functions ===========================================
-   /**
-     * Get number of bytes sent.
-     *
-     * @return Number of bytes sent.
-     */
-   inline card64 getBytesSent() const;
-
-   /**
-     * Get number of bytes received.
-     *
-     * @return Number of bytes received.
-     */
-   inline card64 getBytesReceived() const;
-
-   /**
-     * Reset number of bytes sent.
-     */
-   inline void resetBytesSent();
-
-   /**
-     * Reset number of bytes received.
-     */
-   inline void resetBytesReceived();
-
-
    // ====== Socket control functions =======================================
    /**
      * Check, if socket is ready.
@@ -461,10 +431,10 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      * @param trafficClass Traffic class for packet.
      * @return Bytes sent or error code < 0.
      */
-   ssize_t send(const void*    buffer,
-                const size_t   length,
-                const cardinal flags        = 0,
-                const card8    trafficClass = 0x00);
+   ssize_t send(const void*   buffer,
+                const size_t  length,
+                const integer flags        = 0,
+                const card8   trafficClass = 0x00);
 
    /**
      * Wrapper for sendto().
@@ -479,7 +449,7 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      */
    ssize_t sendTo(const void*          buffer,
                   const size_t         length,
-                  const cardinal       flags,
+                  const integer        flags,
                   const SocketAddress& receiver,
                   const card8          trafficClass = 0x00);
 
@@ -492,7 +462,7 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      * @return Result of sendmsg() call.
      */
    ssize_t sendMsg(const struct msghdr* msg,
-                   const cardinal       flags,
+                   const integer        flags,
                    const card8          trafficClass = 0x00);
 
    /**
@@ -510,12 +480,12 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      *
      * @param buffer Buffer to read data to.
      * @param length Maximum length of data to be received.
-     * @param flags Flags for recv().
+     * @param flags Flags for recvmsg().
      * @return Bytes read or error code < 0.
      */
-   inline ssize_t receive(void*          buffer,
-                          const size_t   length,
-                          const cardinal flags = 0);
+   inline ssize_t receive(void*        buffer,
+                          const size_t length,
+                          integer&     flags);
 
    /**
      * Wrapper for recvfrom().
@@ -523,24 +493,24 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      * @param buffer Buffer to receive data to.
      * @param length Maximum length of data to be received.
      * @param sender Address to store sender's address.
-     * @param flags Flags for recvfrom().
+     * @param flags Flags for recvmsg().
      * @return Bytes received or error code < 0.
      */
    ssize_t receiveFrom(void*          buffer,
                        const size_t   length,
                        SocketAddress& sender,
-                       const cardinal flags = 0);
+                       integer&       flags);
 
    /**
      * Wrapper for recvmsg().
      *
      * @param msg Message.
-     * @param flags Flags.
+     * @param flags Flags for recvmsg().
      * @param internalCall Internal usage only; set to false.
      * @return Result of recvmsg() call.
      */
    ssize_t receiveMsg(struct msghdr* msg,
-                      const cardinal flags,
+                      const integer  flags,
                       const bool     internalCall = false);
 
    /**
@@ -566,11 +536,10 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
      * Wrapper for fcntl().
      *
      * @param cmd Command.
-     * @param arg Argument.
      * @param lock Lock.
      * @return Result of fcntl() call.
      */
-   inline integer fcntl(const integer cmd, long arg, struct flock* lock);
+   inline integer fcntl(const integer cmd, struct flock* lock);
 
    /**
      * Wrapper for ioctl().
@@ -799,7 +768,7 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
    ssize_t recvFrom(int              fd,
                     void*            buf,
                     const size_t     len,
-                    const integer    flags,
+                    integer&         flags,
                     struct sockaddr* addr,
                     socklen_t*       addrlen);
    bool multicastMembership(const SocketAddress& address,
@@ -810,17 +779,15 @@ O     * Bind socket to one or more given addresses. If no addresses are given,
                                sockaddr*               packedArray);
 
 
-   card64    BytesSent;
-   card64    BytesReceived;
-   card32    SendFlow;
-   card32    ReceivedFlow;
-   cardinal  Backlog;
-   integer   LastError;
    int       SocketDescriptor;
-   sockaddr* Destination;
-   integer   CommunicationDomain;
+   integer   Family;
    integer   Type;
    integer   Protocol;
+   card32    SendFlow;
+   card32    ReceivedFlow;
+   integer   LastError;
+   cardinal  Backlog;
+   sockaddr* Destination;
 };
 
 
