@@ -1,33 +1,33 @@
 /*
- *  $Id: internetaddress.cc 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Internet Address Implementation
@@ -43,15 +43,19 @@
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/utsname.h>
+#include <net/if.h>
 #include <arpa/nameser.h>
 #include <ctype.h>
 
 
-
 // Print note, if IPv6 is not available.
 // #ifdef PRINT_NOIPV6_NOTE
 
 
+// Check, if IPv6 is available on this host.
+bool InternetAddress::UseIPv6 = checkIPv6();
+
+
 
 // ###### Internet address constructor ######################################
 InternetAddress::InternetAddress()
@@ -152,10 +156,11 @@ InternetAddress::InternetAddress(const String& hostName,
 InternetAddress::InternetAddress(const PortableAddress& address)
 {
    for(cardinal i = 0;i < 8;i++) {
-      Host[i] = address.Host[i];
+      AddrSpec.Host16[i] = address.Host[i];
    }
-   Port  = address.Port;
-   Valid = true;
+   Port    = address.Port;
+   ScopeID = 0;
+   Valid   = true;
    setPrintFormat(PF_Default);
 }
 
@@ -175,7 +180,7 @@ InternetAddress::InternetAddress(const card16 port)
 
 
 // ###### Internet address constructor ######################################
-InternetAddress::InternetAddress(sockaddr* address, const socklen_t length)
+InternetAddress::InternetAddress(const sockaddr* address, const socklen_t length)
 {
    init(address,length);
 }
@@ -199,9 +204,10 @@ void InternetAddress::setPort(const card16 port)
 void InternetAddress::reset()
 {
    for(cardinal i = 0;i < 8;i++) {
-      Host[i] = 0x0000;
+      AddrSpec.Host16[i] = 0x0000;
    }
-   Valid = true;
+   ScopeID = 0;
+   Valid   = true;
    setPort(0);
    setPrintFormat(PF_Default);
 }
@@ -236,9 +242,10 @@ void InternetAddress::init(const InternetAddress& address)
 {
    Port = address.Port;
    for(cardinal i = 0;i < 8;i++) {
-      Host[i] = address.Host[i];
+      AddrSpec.Host16[i] = address.AddrSpec.Host16[i];
    }
-   Valid = address.Valid;
+   ScopeID = address.ScopeID;
+   Valid   = address.Valid;
    setPrintFormat(address.getPrintFormat());
 }
 
@@ -247,9 +254,10 @@ void InternetAddress::init(const InternetAddress& address)
 void InternetAddress::init(const card16 port)
 {
    for(cardinal i = 0;i < 8;i++) {
-      Host[i] = 0x0000;
+      AddrSpec.Host16[i] = 0x0000;
    }
-   Valid = true;
+   ScopeID = 0;
+   Valid   = true;
    setPort(port);
    setPrintFormat(PF_Default);
 }
@@ -259,21 +267,23 @@ void InternetAddress::init(const card16 port)
 void InternetAddress::init(const String& hostName, const card16 port)
 {
    card16   address[8];
-   cardinal length = getHostByName(hostName.getData(),(card16*)&address);
+   card16   scopeID;
+   cardinal length = getHostByName(hostName.getData(),(card16*)&address,&scopeID);
 
    Valid = true;
    setPort(port);
    setPrintFormat(PF_Default);
+   ScopeID = scopeID;
    switch(length) {
       case 4:
-          for(cardinal i = 0;i < 5;i++) {
-             Host[i] = 0x0000;
-          }
-          Host[5] = 0xffff;
-          memcpy((char*)&Host[6],&address,4);
-        break;
+         for(cardinal i = 0;i < 5;i++) {
+            AddrSpec.Host16[i] = 0x0000;
+         }
+         AddrSpec.Host16[5] = 0xffff;
+         memcpy((char*)&AddrSpec.Host16[6],&address,4);
+       break;
       case 16:
-         memcpy((char*)&Host,&address,16);
+         memcpy((char*)&AddrSpec.Host16,&address,16);
        break;
       default:
         reset();
@@ -287,16 +297,17 @@ void InternetAddress::init(const String& hostName, const card16 port)
 void InternetAddress::init(const PortableAddress& address)
 {
    for(cardinal i = 0;i < 8;i++) {
-      Host[i] = address.Host[i];
+      AddrSpec.Host16[i] = address.Host[i];
    }
-   Port  = address.Port;
-   Valid = true;
+   Port    = address.Port;
+   ScopeID = 0;
+   Valid   = true;
    setPrintFormat(PF_Default);
 }
 
 
 // ###### Initialize internet address from system's sockaddr structure ######
-void InternetAddress::init(sockaddr* address, const socklen_t length)
+void InternetAddress::init(const sockaddr* address, const socklen_t length)
 {
    setSystemAddress(address,length);
    setPrintFormat(PF_Default);
@@ -309,7 +320,7 @@ InternetAddress InternetAddress::getLocalAddress(const InternetAddress& peer)
    InternetAddress address;
 
    int sd = ext_socket((UseIPv6 == true) ? AF_INET6 : AF_INET,SOCK_DGRAM,IPPROTO_UDP);
-   if(socket >= 0) {
+   if(sd >= 0) {
       sockaddr_storage socketAddress;
       socklen_t        socketAddressLength =
                           peer.getSystemAddress((sockaddr*)&socketAddress,SocketAddress::MaxSockLen,
@@ -344,8 +355,8 @@ String InternetAddress::getAddressString(const cardinal format) const
    addressString[0] = 0x00;
    if(!((format & PF_Hostname) || (format & PF_Address))) {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: InternetAddress::getAddressString() - "
-              "Set PF_Hostname or PF_Address before printing!" << endl;
+      std::cerr << "WARNING: InternetAddress::getAddressString() - "
+                   "Set PF_Hostname or PF_Address before printing!" << std::endl;
 #endif
       return(String("(check print format)"));
    }
@@ -394,14 +405,14 @@ String InternetAddress::getAddressString(const cardinal format) const
          // ====== Print IPv6 hex notation ==================================
          const cardinal l0 = strlen(addressString);
          for(cardinal i = 0;i < (cardinal)length;i++) {
-            card16 value = ntohs(Host[i]);
+            const card16 value = ntohs(AddrSpec.Host16[i]);
             if((value != 0) || (compressed == true) || (i == 7)) {
                snprintf((char*)&str,sizeof(str),"%x",value);
             }
             else {
                cardinal j;
                for(j = i + 1;j < 8;j++) {
-                  if(Host[j] != 0) {
+                  if(AddrSpec.Host16[j] != 0) {
                      break;
                   }
                }
@@ -417,7 +428,7 @@ String InternetAddress::getAddressString(const cardinal format) const
                   }
                   compressed = true;
                   i = j - 1;
-               }                        
+               }
             }
             strcat((char*)&addressString,(char*)&str);
             if(i < 7) {
@@ -427,10 +438,10 @@ String InternetAddress::getAddressString(const cardinal format) const
          if(addressString[l0 + 1] == 0x00) { // Any-Adress "::"
             strcat((char*)&addressString,":");
          }
-               
+
          // ====== Print embedded IPv4 address in IPv4 notation =============
          if(length == 6) {
-            card32 a = ntohl(*((card32*)&Host[6]));
+            card32 a = ntohl(AddrSpec.Host32[3]);
             snprintf((char*)&str,sizeof(str),
                      "%d.%d.%d.%d",(a & 0xff000000) >> 24,
                                    (a & 0x00ff0000) >> 16,
@@ -438,7 +449,20 @@ String InternetAddress::getAddressString(const cardinal format) const
                                    (a & 0x000000ff));
             strcat((char*)&addressString,(char*)&str);
          }
-               
+
+         // ====== Interface for link-local address =========================
+         if((isIPv6()) && (isLinkLocal())) {
+            strcat((char*)&addressString,"%");
+            char        ifnamebuffer[IFNAMSIZ];
+            const char* ifname = if_indextoname(ScopeID, (char*)&ifnamebuffer);
+            if(ifname != NULL) {
+               strcat((char*)&addressString,ifname);
+            }
+            else {
+               strcat((char*)&addressString,"(BAD!)");
+            }
+         }
+
          // ====== Add port number ==========================================
          if(!(format & PF_HidePort)) {
             snprintf((char*)&str,sizeof(str),"]:%d",ntohs(Port));
@@ -446,7 +470,7 @@ String InternetAddress::getAddressString(const cardinal format) const
          }
       }
       else {
-         card32 a = ntohl(*((card32*)&Host[6]));
+         card32 a = ntohl(AddrSpec.Host32[3]);
          if(!(format & PF_HidePort)) {
             snprintf((char*)&addressString,sizeof(addressString),
                      "%d.%d.%d.%d:%d",(a & 0xff000000) >> 24,
@@ -485,22 +509,26 @@ cardinal InternetAddress::getSystemAddress(sockaddr*       buffer,
       case AF_INET6: {
          sockaddr_in6* address = (sockaddr_in6*)buffer;
          if(sizeof(sockaddr_in6) <= (size_t)length) {
-            address->sin6_family   = AF_INET6;
+            address->sin6_family = AF_INET6;
+#ifdef HAVE_SIN6_LEN
+            address->sin6_len = sizeof(sockaddr_in6);
+#endif
             address->sin6_flowinfo = 0;
             address->sin6_port     = Port;
+            address->sin6_scope_id = ScopeID;
 #if (SYSTEM == OS_Linux)
-            memcpy((char*)&address->sin6_addr.s6_addr16[0],(char*)&Host,16);
+            memcpy((char*)&address->sin6_addr.s6_addr16[0],(char*)&AddrSpec.Host16,16);
 #elif (SYSTEM == OS_SOLARIS)
-            memcpy((char*)&address->sin6_addr._S6_un._S6_u8[0],(char*)&Host,16);
+            memcpy((char*)&address->sin6_addr._S6_un._S6_u8[0],(char*)&AddrSpec.Host16,16);
 #else
-            memcpy((char*)&address->sin6_addr.__u6_addr.__u6_addr16[0],(char*)&Host,16);
+            memcpy((char*)&address->sin6_addr.__u6_addr.__u6_addr16[0],(char*)&AddrSpec.Host16,16);
 #endif
             return(sizeof(sockaddr_in6));
          }
          else {
 #ifndef DISABLE_WARNINGS
-            cerr << "WARNING: InternetAddress::getSystemInternetAddress() - "
-                    "Buffer size too low for AF_INET6!" << endl;
+            std::cerr << "WARNING: InternetAddress::getSystemInternetAddress() - "
+                         "Buffer size too low for AF_INET6!" << std::endl;
 #endif
          }
         }
@@ -509,24 +537,27 @@ cardinal InternetAddress::getSystemAddress(sockaddr*       buffer,
          sockaddr_in* address = (sockaddr_in*)buffer;
          if(sizeof(sockaddr_in) <= (size_t)length) {
             address->sin_family = AF_INET;
+#ifdef HAVE_SIN_LEN
+            address->sin_len = sizeof(sockaddr_in);
+#endif
             if(isIPv4()) {
                address->sin_port = Port;
-               memcpy((char*)&address->sin_addr.s_addr,(char*)&Host[6],4);
+               memcpy((char*)&address->sin_addr.s_addr,(char*)&AddrSpec.Host16[6],4);
                return(sizeof(sockaddr_in));
             }
          }
          else {
 #ifndef DISABLE_WARNINGS
-            cerr << "WARNING: InternetAddress::getSystemInternetAddress() - "
-                    "Buffer size too low for AF_INET!" << endl;
+            std::cerr << "WARNING: InternetAddress::getSystemInternetAddress() - "
+                         "Buffer size too low for AF_INET!" << std::endl;
 #endif
          }
         }
        break;
       default:
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: InternetAddress::getSystemInternetAddress() - Unknown type "
-              << newType << "!" << endl;
+         std::cerr << "WARNING: InternetAddress::getSystemInternetAddress() - Unknown type "
+                   << newType << "!" << std::endl;
 #endif
        break;
    }
@@ -535,31 +566,32 @@ cardinal InternetAddress::getSystemAddress(sockaddr*       buffer,
 
 
 // ###### Initialize internet address from sockaddr structure ###############
-bool InternetAddress::setSystemAddress(sockaddr* address, const socklen_t length)
+bool InternetAddress::setSystemAddress(const sockaddr* address, const socklen_t length)
 {
-   sockaddr_in* address4 = (sockaddr_in*)address;
+   const sockaddr_in* address4 = (const sockaddr_in*)address;
    Port = address4->sin_port;
 
    switch(address4->sin_family) {
       case AF_INET:
          for(cardinal i = 0;i < 5;i++) {
-            Host[i] = 0x0000;
+            AddrSpec.Host16[i] = 0x0000;
          }
-         Host[5] = 0xffff;
-         memcpy((char*)&Host[6],(char*)&address4->sin_addr.s_addr,4);
+         AddrSpec.Host16[5] = 0xffff;
+         memcpy((char*)&AddrSpec.Host16[6],(const char*)&address4->sin_addr.s_addr,4);
          Valid = true;
          return(true);
        break;
       case AF_INET6: {
-         sockaddr_in6* address6 = (sockaddr_in6*)address;
+         const sockaddr_in6* address6 = (const sockaddr_in6*)address;
 #if (SYSTEM == OS_Linux)
-         memcpy((char*)&Host,(char*)&address6->sin6_addr.s6_addr16[0],16);
+         memcpy((char*)&AddrSpec.Host16,(const char*)&address6->sin6_addr.s6_addr16[0],16);
 #elif (SYSTEM == OS_SOLARIS)
-         memcpy((char*)&Host,(char*)&address6->sin6_addr._S6_un._S6_u8[0],16);
+         memcpy((char*)&AddrSpec.Host16,(const char*)&address6->sin6_addr._S6_un._S6_u8[0],16);
 #else
-         memcpy((char*)&Host,(char*)&address6->sin6_addr.__u6_addr.__u6_addr8[0],16);
+         memcpy((char*)&AddrSpec.Host16,(const char*)&address6->sin6_addr.__u6_addr.__u6_addr8[0],16);
 #endif
-         Valid = true;
+         ScopeID = address6->sin6_scope_id;
+         Valid   = true;
          return(true);
         }
        break;
@@ -578,14 +610,11 @@ bool InternetAddress::checkIPv6()
    int result = socket(AF_INET6,SOCK_DGRAM,0);
    if(result != -1) {
       close(result);
-#if (SYSTEM != OS_Darwin)
-      _res.options |= RES_USE_INET6;
-#endif
       return(true);
    }
    else {
 #ifdef PRINT_NOIPV6_NOTE
-      cerr << "IPv6 is unsupported on this host!" << endl;
+      std::cerr << "IPv6 is unsupported on this host!" << std::endl;
 #endif
       return(false);
    }
@@ -593,9 +622,12 @@ bool InternetAddress::checkIPv6()
 
 
 // ###### Get host address by name ##########################################
-cardinal InternetAddress::getHostByName(const String& hostName, card16* myadr)
+cardinal InternetAddress::getHostByName(const String& hostName, card16* myadr, card16* myscope)
 {
    // ====== Check for null address =========================================
+   if(myscope) {
+      *myscope = 0;
+   }
    if(hostName.isNull()) {
       for(cardinal i = 0;i < 8;i++) myadr[i] = 0;
       if(UseIPv6) {
@@ -611,7 +643,13 @@ cardinal InternetAddress::getHostByName(const String& hostName, card16* myadr)
    addrinfo* res = NULL;
    memset((char*)&hints,0,sizeof(hints));
    hints.ai_socktype = SOCK_DGRAM;
-   hints.ai_family   = (UseIPv6 == true) ? AF_UNSPEC : AF_INET;
+   if(UseIPv6 == true) {
+      hints.ai_family = AF_UNSPEC;
+      hints.ai_flags  = AI_ADDRCONFIG;
+   }
+   else {
+      hints.ai_family = AF_INET;
+   }
    const char* name = hostName.getData();
 
    // Avoid DNS lookups for IP addresses
@@ -651,8 +689,18 @@ cardinal InternetAddress::getHostByName(const String& hostName, card16* myadr)
        break;
       case AF_INET6: {
              const sockaddr_in6* adr = (const sockaddr_in6*)res->ai_addr;
-             memcpy((char*)myadr,(const char*)&adr->sin6_addr,16);
-             result = 16;
+             if((IN6_IS_ADDR_LINKLOCAL(&adr->sin6_addr)) &&
+                (adr->sin6_scope_id == 0)) {
+                // Link-local without scope!
+                result = 0;
+             }
+             else {
+                memcpy((char*)myadr,(const char*)&adr->sin6_addr,16);
+                if(myscope) {
+                   *myscope = adr->sin6_scope_id;
+                }
+                result = 16;
+             }
           }
        break;
       default:
