@@ -1,33 +1,33 @@
 /*
- *  $Id: internetaddress.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Internet Address Implementation
@@ -53,9 +53,9 @@
   * This class manages an internet address.
   *
   * @short   Socket Address
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
-  */            
+  */
 class InternetAddress : virtual public SocketAddress
 {
    // ====== Constructors/Destructor ========================================
@@ -78,7 +78,7 @@ class InternetAddress : virtual public SocketAddress
      *
      * @param address Address string.
      */
-   InternetAddress(const String& address);      
+   InternetAddress(const String& address);
 
    /**
      * Constructor for a internet address given by host name and port.
@@ -103,7 +103,7 @@ class InternetAddress : virtual public SocketAddress
      * @param address sockaddr.
      * @param length Length of sockaddr (sizeof(sockaddr_in) or sizeof(sockaddr_in6)).
      */
-   InternetAddress(sockaddr* address, socklen_t length);
+   InternetAddress(const sockaddr* address, const socklen_t length);
 
    /**
      * Destructor.
@@ -134,7 +134,7 @@ class InternetAddress : virtual public SocketAddress
      *
      * @param hostName Host name.
      * @param port Port number.
-     */    
+     */
    void init(const String& hostName, const card16 port);
 
    /**
@@ -151,7 +151,7 @@ class InternetAddress : virtual public SocketAddress
      * @param address sockaddr.
      * @param length Length of sockaddr (sizeof(sockaddr_in) or sizeof(sockaddr_in6)).
      */
-   void init(sockaddr* address, socklen_t length);
+   void init(const sockaddr* address, const socklen_t length);
 
 
    // ====== Operators ======================================================
@@ -250,27 +250,6 @@ class InternetAddress : virtual public SocketAddress
      */
    inline bool isLoopback() const;
 
-   /**
-     * Check, if internet address is class A IPv4-compatible or IPv4-mapped.
-     *
-     * @return true or false.
-     */
-   inline bool isClassA() const;
-
-   /**
-     * Check, if internet address is class B IPv4-compatible or IPv4-mapped.
-     *
-     * @return true or false.
-     */
-   inline bool isClassB() const;
-
-   /**
-     * Check, if internet address is class C IPv4-compatible or IPv4-mapped.
-     *
-     * @return true or false.
-     */
-   inline bool isClassC() const;
-
    /**
      * Check, if internet address is link local (IPv6) or 127.x.y.z (IPv4).
      *
@@ -362,7 +341,7 @@ class InternetAddress : virtual public SocketAddress
      * getSystemAddress() implementation of SocketAddress
      *
      * @see SocketAddress#getSystemAddress
-     */            
+     */
    cardinal getSystemAddress(sockaddr*       buffer,
                              const socklen_t length,
                              const cardinal  type) const;
@@ -372,7 +351,7 @@ class InternetAddress : virtual public SocketAddress
      *
      * @see SocketAddress#setSystemAddress
      */
-   bool setSystemAddress(sockaddr* address, const socklen_t length);
+   bool setSystemAddress(const sockaddr* address, const socklen_t length);
 
 
    // ====== IPv6 support functions ========================================
@@ -383,9 +362,10 @@ class InternetAddress : virtual public SocketAddress
      *
      * @param name Host name.
      * @param myadr Storage space to save a IPv6 address (16 bytes).
+     * @param myscope Storage space to save IPv6 link local scope ID to (or NULL).
      * @param length Length of the address saved in myaddr or 0 in case of failure.
      */
-   static cardinal getHostByName(const String& name, card16* myadr);
+   static cardinal getHostByName(const String& name, card16* myadr, card16* myscope = NULL);
 
    /**
      * Get port number for given service (e.g. http).
@@ -543,13 +523,22 @@ class InternetAddress : virtual public SocketAddress
      * Host address in network byte order. IPv4 addresses are converted to
      * IPv4-mapped IPv6 addresses.
      */
-   card16 Host[8];
+   union {
+      card16   Host16[8];
+      card32   Host32[4];
+      in6_addr Address;
+   } AddrSpec;
 
    /**
      * Port number.
      */
    card16 Port;
 
+   /**
+     * Scope ID.
+     */
+   card16 ScopeID;
+
    /**
      * Is address valid?
      */
