@@ -1,33 +1,33 @@
 /*
- *  $Id: sctpassociation.cc 681 2005-08-03 11:23:45Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: SCTP Association
@@ -36,16 +36,19 @@
 
 
 #include "tdsystem.h"
+#include "tools.h"
 #include "sctpassociation.h"
 #include "sctpsocketmaster.h"
 
 
+
 // #define PRINT_PREESTABLISHMENT_SEND
 // #define PRINT_SHUTDOWN
 // #define PRINT_SOCKTYPE
 // #define PRINT_RTOMAX
 
 
+
 // ###### Constructor #######################################################
 SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
                                  const unsigned int associationID,
@@ -87,7 +90,7 @@ SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
 
    if(!udpLike) {
 #ifdef PRINT_SOCKTYPE
-      cout << "SCTPAssociation::SCTPAssociation() - Initializing TCP-like socket" << endl;
+      std::cout << "SCTPAssociation::SCTPAssociation() - Initializing TCP-like socket" << std::endl;
 #endif
       ShutdownCompleteCondition.addParent(&ExceptUpdateCondition);
       EstablishCondition.addParent(&WriteUpdateCondition);
@@ -95,12 +98,12 @@ SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
    }
 #ifdef PRINT_SOCKTYPE
    else {
-      cout << "SCTPAssociation::SCTPAssociation() - Initializing UDP-like socket" << endl;
+      std::cout << "SCTPAssociation::SCTPAssociation() - Initializing UDP-like socket" << std::endl;
    }
 #endif
 
    SCTPSocketMaster::MasterInstance.lock();
-   Socket->AssociationList.insert(pair<unsigned int, SCTPAssociation*>(AssociationID,this));
+   Socket->AssociationList.insert(std::pair<unsigned int, SCTPAssociation*>(AssociationID,this));
    SCTPSocketMaster::MasterInstance.unlock();
 }
 
@@ -111,42 +114,49 @@ SCTPAssociation::~SCTPAssociation()
    SCTPSocketMaster::MasterInstance.lock();
    if(AssociationID == 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SCTPAssociation::~SCTPAssociation() - "
-              "AssociationID is 0! Destructor called twice?!" << endl;
+      std::cerr << "ERROR: SCTPAssociation::~SCTPAssociation() - "
+                   "AssociationID is 0! Destructor called twice?!" << std::endl;
 #endif
+      ::abort();
       return;
    }
 
    // ====== Do shutdown ====================================================
    if(!ShutdownCompleteNotification) {
 #ifdef PRINT_SHUTDOWN
-      cout << "Active shutdown of association #" << AssociationID << " started..." << endl;
+      std::cout << "Active shutdown of association #" << AssociationID << " started..." << std::endl;
 #endif
       SCTPSocketMaster::delayedDeleteAssociation(Socket->getID(),AssociationID);
       shutdown();
 #ifdef PRINT_SHUTDOWN
-      cout << "Active shutdown of association #" << AssociationID << " complete!" << endl;
+      std::cout << "Active shutdown of association #" << AssociationID << " complete!" << std::endl;
 #endif
    }
    else {
 #ifdef PRINT_SHUTDOWN
-      cout << "Passive shutdown of association #" << AssociationID << "!" << endl;
+      std::cout << "Passive shutdown of association #" << AssociationID << "!" << std::endl;
 #endif
-      sctp_deleteAssociation(AssociationID);
+
+      if(sctp_deleteAssociation(AssociationID) != SCTP_SUCCESS) {
+#ifndef DISABLE_WARNINGS
+         std::cerr << "INTERNAL ERROR: SCTPAssociation::~SCTPAssociation() - sctp_deleteAssociation() failed!" << std::endl;
+#endif
+         ::abort();
+      }
    }
 
-   // ====== Remove association from list ================================
-   multimap<unsigned int, SCTPAssociation*>::iterator iterator =
+   // ====== Remove association from list ===================================
+   std::multimap<unsigned int, SCTPAssociation*>::iterator iterator =
       Socket->AssociationList.find(AssociationID);
    if(iterator != Socket->AssociationList.end()) {
       Socket->AssociationList.erase(iterator);
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "INTERNAL ERROR: SCTPAssociation::~SCTPAssociation() - "
-              "Erase of association #" << AssociationID << " failed!" << endl;
-      ::exit(1);
+      std::cerr << "INTERNAL ERROR: SCTPAssociation::~SCTPAssociation() - "
+                   "Erase of association #" << AssociationID << " from association list failed!" << std::endl;
 #endif
+      ::abort();
    }
 
    // ====== Clear ID, to make finding dangling references easier ===========
@@ -187,13 +197,11 @@ bool SCTPAssociation::getLocalAddresses(SocketAddress**& addressArray)
 // ###### Get remote address ################################################
 bool SCTPAssociation::getRemoteAddresses(SocketAddress**& addressArray)
 {
-   if(addressArray != NULL) {
-      addressArray = NULL;
-   }
    bool                    result = true;
    SCTP_Association_Status status;
-   addressArray = NULL;
+   int                     ok;
 
+   addressArray = NULL;
    SCTPSocketMaster::MasterInstance.lock();
 
    if(sctp_getAssocStatus(AssociationID,&status) == 0) {
@@ -219,23 +227,27 @@ bool SCTPAssociation::getRemoteAddresses(SocketAddress**& addressArray)
 #endif
 
          SCTP_Path_Status pathStatus;
-         if(sctp_getPathStatus(AssociationID,index,&pathStatus) != 0) {
+         ok = sctp_getPathStatus(AssociationID, index, &pathStatus);
+         if(ok != SCTP_SUCCESS) {
 #ifndef DISABLE_WARNINGS
-            cerr << "INTERNAL ERROR: SCTPAssociation::getRemoteAddress() - sctp_getPathStatus() failure!"
-                 << endl;
-            ::exit(1);
+            std::cerr << "WARNING: SCTPAssociation::getRemoteAddress() - sctp_getPathStatus() failure!" << std::endl
+                      << "return code: " << ok << std::endl;
 #endif
+            SocketAddress::deleteAddressList(addressArray);
+            result = false;
+            break;
          }
          else {
             addressArray[i] = SocketAddress::createSocketAddress(
-                                 0,(char*)&pathStatus.destinationAddress,status.destPort);
+                                 0, (char*)&pathStatus.destinationAddress,status.destPort);
             if(addressArray[i] == NULL) {
 #ifndef DISABLE_WARNINGS
-               cerr << "WARNING: SCTPAssociation::getRemoteAddresses() - Bad address "
-                    << pathStatus.destinationAddress << ", port " << status.destPort << "!" << endl;
+               std::cerr << "WARNING: SCTPAssociation::getRemoteAddresses() - Bad address "
+                         << pathStatus.destinationAddress << ", port " << status.destPort << "!" << std::endl;
 #endif
                SocketAddress::deleteAddressList(addressArray);
                result = false;
+               break;
             }
          }
       }
@@ -388,9 +400,9 @@ bool SCTPAssociation::sendPreEstablishmentPackets()
 #ifdef PRINT_PREESTABLISHMENT_SEND
       char str[256];
       snprintf((char*)&str,sizeof(str),
-                  "A%04d: sendPreEstablishmentPackets() - Sending %u bytes, PPID $%08x, stream %u",
-                  AssociationID, packet->Length, packet->ProtoID, packet->StreamID);
-      cerr << str << endl;
+               "A%04d: sendPreEstablishmentPackets() - Sending %u bytes, PPID $%08x, stream %u",
+               AssociationID, packet->Length, packet->ProtoID, packet->StreamID);
+      std::cerr << str << std::endl;
 #endif
       result = sendTo(packet->Data,
                       packet->Length,
@@ -402,7 +414,7 @@ bool SCTPAssociation::sendPreEstablishmentPackets()
                       NULL);
       if(result == (ssize_t)packet->Length) {
 #ifdef PRINT_PREESTABLISHMENT_SEND
-         cerr << "Successfully sent packet" << endl;
+         std::cerr << "Successfully sent packet" << std::endl;
 #endif
          FirstPreEstablishmentPacket = packet->Next;
          if(LastPreEstablishmentPacket == packet) {
@@ -414,7 +426,7 @@ bool SCTPAssociation::sendPreEstablishmentPackets()
       }
       else {
 #ifdef PRINT_PREESTABLISHMENT_SEND
-         cerr << "Sending failed" << endl;
+         std::cerr << "Sending failed" << std::endl;
 #endif
          return(false);
       }
@@ -461,7 +473,7 @@ bool SCTPAssociation::getAssocStatus(
 #ifdef PRINT_RTOMAX
          char str[256];
          snprintf((char*)&str,sizeof(str),"getAssocStatus() - Replying RTOMax=%d instead of InitTimeout=%d\n",assocStatus.rtoMax,RTOMax);
-         cerr << str << endl;
+         std::cerr << str << std::endl;
 #endif
          assocStatus.rtoMax = RTOMax;
       }
@@ -479,12 +491,12 @@ bool SCTPAssociation::setAssocStatus(
 #ifdef PRINT_RTOMAX
       char str[256];
       snprintf((char*)&str,sizeof(str),"setAssocStatus() - Saving RTOMax=%d, using InitTimeout=%d\n",assocStatus.rtoMax,InitTimeout);
-      cerr << str << endl;
+      std::cerr << str << std::endl;
 #endif
       newStatus.rtoMax = InitTimeout;
       RTOMax = assocStatus.rtoMax;
    }
-   return(Socket->setAssocStatus(AssociationID,assocStatus));
+   return(Socket->setAssocStatus(AssociationID,newStatus));
 }
 
 
@@ -553,16 +565,16 @@ bool SCTPAssociation::getDefaultStreamTimeout(const unsigned short streamID,
 
 
 // ###### Get path parameters ###############################################
-bool SCTPAssociation::getPathParameters(const struct SocketAddress* address,
-                                        SCTP_PathStatus&            pathParameters)
+bool SCTPAssociation::getPathParameters(const SocketAddress* address,
+                                        SCTP_PathStatus&     pathParameters)
 {
    return(Socket->getPathParameters(AssociationID,address,pathParameters));
 }
 
 
 // ###### Set path parameters ###############################################
-bool SCTPAssociation::setPathParameters(const struct SocketAddress* address,
-                                        const SCTP_PathStatus&      pathParameters)
+bool SCTPAssociation::setPathParameters(const SocketAddress*   address,
+                                        const SCTP_PathStatus& pathParameters)
 {
    return(Socket->setPathParameters(AssociationID,address,pathParameters));
 }
