@@ -1,33 +1,33 @@
 /*
- *  $Id: sctpnotificationqueue.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Chunk Arrival Queue
@@ -46,14 +46,14 @@
 
 
 
-#define SCTP_RECVDATAIOEVNT        (1 << 0)
-#define SCTP_RECVASSOCEVNT         (1 << 1)
-#define SCTP_RECVPADDREVNT         (1 << 2)
-#define SCTP_RECVPEERERR           (1 << 3)
-#define SCTP_RECVSENDFAILEVNT      (1 << 4)
-#define SCTP_RECVSHUTDOWNEVNT      (1 << 5)
-#define SCTP_RECVADAPIONINDICATION (1 << 6)
-#define SCTP_RECVPDEVNT            (1 << 7)
+#define SCTP_RECVDATAIOEVNT           (1 << 0)
+#define SCTP_RECVASSOCEVNT            (1 << 1)
+#define SCTP_RECVPADDREVNT            (1 << 2)
+#define SCTP_RECVPEERERR              (1 << 3)
+#define SCTP_RECVSENDFAILEVNT         (1 << 4)
+#define SCTP_RECVSHUTDOWNEVNT         (1 << 5)
+#define SCTP_RECVADAPTATIONINDICATION (1 << 6)
+#define SCTP_RECVPDEVNT               (1 << 7)
 
 
 
@@ -61,7 +61,7 @@
   * SCTP Notification
   *
   * @short   SCTP Notification
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   */
 struct SCTPNotification
@@ -114,7 +114,7 @@ enum UpdateConditionType {
   * This class is a queue for SCTP notifications.
   *
   * @short   SCTP Notification Queue
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   */
 class SCTPNotificationQueue
