@@ -1,33 +1,33 @@
 /*
- *  $Id: sctpsocket.h 664 2005-07-28 12:18:32Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: SCTP Socket
@@ -46,16 +46,18 @@
 #include "sctpassociation.h"
 #include "sctpnotificationqueue.h"
 
-
 #include <sctp.h>
 #include <map>
 
 
+class SCTPAssociation;
+
+
 /**
   * This class manages a SCTP socket (SCTP instance).
   *
   * @short   SCTP Socket
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   */
 class SCTPSocket
@@ -267,7 +269,7 @@ class SCTPSocket
    int sendTo(const char*           buffer,
               const size_t          length,
               const int             flags,
-              const unsigned int    assocID,
+              unsigned int&         assocID,
               const unsigned short  streamID,
               const unsigned int    protoID,
               const unsigned int    timeToLive,
@@ -396,9 +398,9 @@ class SCTPSocket
      * @param pathParameters Reference to store path parameters.
      * @return true, if successful; false otherwise.
      */
-   bool getPathParameters(const unsigned int          assocID,
-                          const struct SocketAddress* address,
-                          SCTP_PathStatus&            pathParameters);
+   bool getPathParameters(const unsigned int   assocID,
+                          const SocketAddress* address,
+                          SCTP_PathStatus&     pathParameters);
 
    /**
      * Set path parameters.
@@ -408,9 +410,9 @@ class SCTPSocket
      * @param pathParameters Path parameters.
      * @return true, if successful; false otherwise.
      */
-   bool setPathParameters(const unsigned int          assocID,
-                          const struct SocketAddress* address,
-                          const SCTP_PathStatus&      pathParameters);
+   bool setPathParameters(const unsigned int     assocID,
+                          const SocketAddress*   address,
+                          const SCTP_PathStatus& pathParameters);
 
    /**
      * Get primary address of given association.
@@ -540,9 +542,9 @@ class SCTPSocket
                     const unsigned int   timeToLive,
                     Condition*           waitCondition,
                     const SocketAddress* pathDestinationAddress);
-   static int getPathIndexForAddress(const unsigned int          assocID,
-                                     const struct SocketAddress* address,
-                                     SCTP_PathStatus&            pathParameters);
+   static int getPathIndexForAddress(const unsigned int   assocID,
+                                     const SocketAddress* address,
+                                     SCTP_PathStatus&     pathParameters);
 
 
    struct IncomingConnection
@@ -552,50 +554,50 @@ class SCTPSocket
       SCTPNotification    Notification;
    };
 
-   SCTPNotificationQueue                    GlobalQueue;
-   Condition                                EstablishCondition;
-   Condition                                ReadUpdateCondition;
-   Condition                                WriteUpdateCondition;
-   Condition                                ExceptUpdateCondition;
+   SCTPNotificationQueue                         GlobalQueue;
+   Condition                                     EstablishCondition;
+   Condition                                     ReadUpdateCondition;
+   Condition                                     WriteUpdateCondition;
+   Condition                                     ExceptUpdateCondition;
 
-   IncomingConnection*                      ConnectionRequests;
-   multimap<unsigned int, SCTPAssociation*> AssociationList;
+   IncomingConnection*                           ConnectionRequests;
+   std::multimap<unsigned int, SCTPAssociation*> AssociationList;
 
-   int                                      InstanceName;
-   unsigned short                           LocalPort;
-   unsigned short                           NoOfInStreams;
-   unsigned short                           NoOfOutStreams;
+   int                                           InstanceName;
+   unsigned short                                LocalPort;
+   unsigned short                                NoOfInStreams;
+   unsigned short                                NoOfOutStreams;
 
-   cardinal                                 Flags;
-   unsigned int                             NotificationFlags;
-   unsigned int                             CorrelationID;
+   cardinal                                      Flags;
+   unsigned int                                  NotificationFlags;
+   unsigned int                                  CorrelationID;
 
-   card64                                   AutoCloseTimeout;
+   card64                                        AutoCloseTimeout;
 
 
    // ====== Private data ===================================================
    private:
    void checkAutoConnect();
    void checkAutoClose();
-   SCTPAssociation* SCTPSocket::findAssociationForDestinationAddress(
-                       multimap<unsigned int, SCTPAssociation*>& list,
+   SCTPAssociation* findAssociationForDestinationAddress(
+                       std::multimap<unsigned int, SCTPAssociation*>& list,
                        const SocketAddress** destinationAddressList);
 
 
-   multimap<unsigned int, SCTPAssociation*> ConnectionlessAssociationList;
-   card8                                    DefaultTrafficClass;
+   std::multimap<unsigned int, SCTPAssociation*> ConnectionlessAssociationList;
+   card8                                         DefaultTrafficClass;
 
-   int                                      Family;
-   bool                                     WriteReady;
-   bool                                     ReadReady;
-   bool                                     HasException;
+   int                                           Family;
+   bool                                          WriteReady;
+   bool                                          ReadReady;
+   bool                                          HasException;
 
-   bool                                     AutoCloseRecursion;
-   bool                                     AutoCloseNewCheckRequired;
+   bool                                          AutoCloseRecursion;
+   bool                                          AutoCloseNewCheckRequired;
 
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19)
-   unsigned int                             NoOfLocalAddresses;
-   unsigned char                            LocalAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN];
+   unsigned int                                  NoOfLocalAddresses;
+   unsigned char                                 LocalAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN];
 #endif
 };
 
