@@ -1,33 +1,33 @@
 /*
- *  $Id: sctpsocketmaster.cc 660 2005-07-22 14:30:13Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: SCTP Socket Master
@@ -37,10 +37,13 @@
 
 
 #include "tdsystem.h"
+#include "tools.h"
 #include "sctpsocketmaster.h"
+#include "extsocketdescriptor.h"
 
 
 #include <signal.h>
+#include <poll.h>
 #if (SYSTEM == OS_SOLARIS)
 #include <sys/types.h>
 #include <unistd.h>
@@ -48,20 +51,41 @@
 #endif
 
 
-
 // #define PRINT_NOTIFICATIONS
 // #define PRINT_ARRIVENOTIFICATION
+// #define PRINT_USERCALLBACK
+// #define PRINT_ASSOC_USECOUNT
+// #define PRINT_RTOMAXRESTORE
 //
 // #define PRINT_PIPE
 // #define PRINT_GC
-// #define PRINT_ASSOC_USECOUNT
-// #define PRINT_RTOMAXRESTORE
 
 // Do not show a warning on initialization failure.
 // #define NO_INITFAIL_WARNING
 
 
 
+// ###### SCTPSocketMaster static attributes ################################
+int                              SCTPSocketMaster::InitializationResult     = -1000;
+int                              SCTPSocketMaster::GarbageCollectionTimerID = -1;
+cardinal                         SCTPSocketMaster::LockLevel                = 0;
+cardinal                         SCTPSocketMaster::OldCancelState           = true;
+card64                           SCTPSocketMaster::LastGarbageCollection;
+std::set<int>                    SCTPSocketMaster::ClosingSockets;
+std::multimap<unsigned int, int> SCTPSocketMaster::ClosingAssociations;
+std::multimap<int, SCTPSocket*>  SCTPSocketMaster::SocketList;
+SCTP_ulpCallbacks                SCTPSocketMaster::Callbacks;
+SCTPSocketMaster                 SCTPSocketMaster::MasterInstance;
+Randomizer                       SCTPSocketMaster::Random;
+int                              SCTPSocketMaster::BreakPipe[2];
+SCTPSocketMaster::UserSocketNotification
+                                 SCTPSocketMaster::BreakNotification;
+
+ExtSocketDescriptor              ExtSocketDescriptorMaster::Sockets[ExtSocketDescriptorMaster::MaxSockets];
+ExtSocketDescriptorMaster        ExtSocketDescriptorMaster::MasterInstance;
+
+
+
 // ###### Library version check #############################################
 static bool versionCheck()
 {
@@ -71,7 +95,7 @@ static bool versionCheck()
    const unsigned int sctplibCompiledVersion = (SCTP_MAJOR_VERSION << 16) | SCTP_MINOR_VERSION;
 
    if(sctplibLinkedVersion != sctplibCompiledVersion) {
-      cerr << "INTERNAL ERROR: sctp.h and linked sctplib library are different!" << endl;
+      std::cerr << "INTERNAL ERROR: sctp.h and linked sctplib library are different!" << std::endl;
       result = false;
    }
 
@@ -80,7 +104,7 @@ static bool versionCheck()
       snprintf((char*)&str, sizeof(str),
                "Compiled = $%04x\nLinked   = $%04x\n",
                sctplibCompiledVersion, sctplibLinkedVersion);
-      cerr << str;
+      std::cerr << str;
    }
 
    return(result);
@@ -113,9 +137,13 @@ SCTPSocketMaster::SCTPSocketMaster()
       int sd;
       sd = socket(AF_INET, SOCK_SEQPACKET, IPPROTO_SCTP);
       if(sd >= 0) {
+#ifndef SCTP_OVER_UDP
          close(sd);
-         cerr << "ERROR: Kernel SCTP seems to be available! You cannout use sctplib and kernel SCTP simultaneously!" << endl;
-         ::exit(1);
+         std::cerr << "ERROR: Kernel SCTP seems to be available! You cannout use sctplib and kernel SCTP simultaneously!" << std::endl;
+         ::abort();
+#else
+         std::cerr << "NOTE: The socket API assumes SCTP over UDP. Kernel SCTP has been found, but this should be okay." << std::endl;
+#endif
       }
 
       InitializationResult = sctp_initLibrary();
@@ -126,7 +154,7 @@ SCTPSocketMaster::SCTPSocketMaster()
 
          if(pipe((int*)&BreakPipe) == 0) {
 #ifdef PRINT_PIPE
-            cout << "Break Pipe: in=" << BreakPipe[0] << " out=" << BreakPipe[1] << endl;
+            std::cout << "Break Pipe: in=" << BreakPipe[0] << " out=" << BreakPipe[1] << std::endl;
 #endif
             int flags = fcntl(BreakPipe[0],F_GETFL,0);
             if(flags != -1) {
@@ -142,7 +170,7 @@ SCTPSocketMaster::SCTPSocketMaster()
                }
                else {
 #ifndef DISABLE_WARNINGS
-                  cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Failed to set Break Pipe to non-blocking mode!" << endl;
+                  std::cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Failed to set Break Pipe to non-blocking mode!" << std::endl;
 #endif
                   close(BreakPipe[0]);
                   close(BreakPipe[1]);
@@ -152,7 +180,7 @@ SCTPSocketMaster::SCTPSocketMaster()
             }
             else {
 #ifndef DISABLE_WARNINGS
-               cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Failed reading Break Pipe flags!" << endl;
+               std::cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Failed reading Break Pipe flags!" << std::endl;
 #endif
                close(BreakPipe[0]);
                close(BreakPipe[1]);
@@ -164,7 +192,7 @@ SCTPSocketMaster::SCTPSocketMaster()
             BreakPipe[0] = -1;
             BreakPipe[1] = -1;
 #ifndef DISABLE_WARNINGS
-            cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Break Pipe not available!" << endl;
+            std::cerr << "WARNING: SCTPSocketMaster::SCTPSocketMaster() - Break Pipe not available!" << std::endl;
 #endif
          }
       }
@@ -172,17 +200,17 @@ SCTPSocketMaster::SCTPSocketMaster()
          BreakPipe[0] = -1;
          BreakPipe[1] = -1;
 #ifndef NO_INITFAIL_WARNING
-         cerr << "ERROR: SCTP Library initialization failed!" << endl;
+         std::cerr << "ERROR: SCTP Library initialization failed!" << std::endl;
          if(getuid() != 0) {
-            cerr << "       You need root permissions to use the SCTP Library!" << endl;
+            std::cerr << "       You need root permissions to use the SCTP Library!" << std::endl;
          }
 #endif
       }
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SCTPSocketMaster::SCTPSocketMaster() - "
-              "Do not try to initialice SCTPSocketMaster singleton twice!" << endl;
+      std::cerr << "ERROR: SCTPSocketMaster::SCTPSocketMaster() - "
+              "Do not try to initialice SCTPSocketMaster singleton twice!" << std::endl;
 #endif
    }
 }
@@ -192,7 +220,7 @@ SCTPSocketMaster::SCTPSocketMaster()
 SCTPSocketMaster::~SCTPSocketMaster()
 {
 #ifdef PRINT_GC
-   cout << "garbageCollection: SocketMaster destructor..." << endl;
+   std::cout << "garbageCollection: SocketMaster destructor..." << std::endl;
 #endif
 
    // ====== Stop master thread =============================================
@@ -208,7 +236,7 @@ SCTPSocketMaster::~SCTPSocketMaster()
       GarbageCollectionTimerID = -1;
    }
 
-   multimap<unsigned int, int>::iterator iterator = ClosingAssociations.begin();
+   std::multimap<unsigned int, int>::iterator iterator = ClosingAssociations.begin();
    while(iterator != ClosingAssociations.end()) {
       // associationGarbageCollection(iterator->first,true) may not be called
       // here, since sctp_abort() directly calls communicationLostNotification(),
@@ -231,7 +259,7 @@ SCTPSocketMaster::~SCTPSocketMaster()
 
    // ====== Remove break pipe ==============================================
    if(BreakPipe[0] != -1) {
-      SCTPSocketMaster::MasterInstance.addUserSocketNotification(&BreakNotification);
+      SCTPSocketMaster::MasterInstance.deleteUserSocketNotification(&BreakNotification);
       close(BreakPipe[0]);
       close(BreakPipe[1]);
       BreakPipe[0] = -1;
@@ -239,7 +267,7 @@ SCTPSocketMaster::~SCTPSocketMaster()
    }
 
 #ifdef PRINT_GC
-   cout << "garbageCollection: SocketMaster destructor completed!" << endl;
+   std::cout << "garbageCollection: SocketMaster destructor completed!" << std::endl;
 #endif
 }
 
@@ -292,14 +320,14 @@ bool SCTPSocketMaster::enableOOTBHandling(const bool enable)
       parameters.sendOotbAborts    = (enable == false) ? 0 : 1;
       if(sctp_setLibraryParameters(&parameters) != SCTP_SUCCESS) {
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Setting of SCTP Library parameters failed!" << endl;
+         std::cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Setting of SCTP Library parameters failed!" << std::endl;
 #endif
          result = false;
       }
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Getting of SCTP Library parameters failed!" << endl;
+      std::cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Getting of SCTP Library parameters failed!" << std::endl;
 #endif
       result = false;
    }
@@ -319,14 +347,14 @@ bool SCTPSocketMaster::enableCRC32(const bool enable)
          (enable == true) ? SCTP_CHECKSUM_ALGORITHM_CRC32C : SCTP_CHECKSUM_ALGORITHM_ADLER32;
       if(sctp_setLibraryParameters(&parameters) != SCTP_SUCCESS) {
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Setting of SCTP Library parameters failed!" << endl;
+         std::cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Setting of SCTP Library parameters failed!" << std::endl;
 #endif
          result = false;
       }
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Getting of SCTP Library parameters failed!" << endl;
+      std::cerr << "WARNING: SCTPSocketMaster::enableOOTBHandling() - Getting of SCTP Library parameters failed!" << std::endl;
 #endif
       result = false;
    }
@@ -340,9 +368,9 @@ void SCTPSocketMaster::delayedDeleteAssociation(const unsigned short instanceID,
                                                 const unsigned int   assocID)
 {
 #ifdef PRINT_GC
-   cout << "delayedDeleteAssociation: A=" << assocID << " I=" << instanceID << endl;
+   std::cout << "delayedDeleteAssociation: A=" << assocID << " I=" << instanceID << std::endl;
 #endif
-   ClosingAssociations.insert(pair<unsigned int, unsigned short>(assocID,instanceID));
+   ClosingAssociations.insert(std::pair<unsigned int, unsigned short>(assocID,instanceID));
 }
 
 
@@ -350,7 +378,7 @@ void SCTPSocketMaster::delayedDeleteAssociation(const unsigned short instanceID,
 void SCTPSocketMaster::delayedDeleteSocket(const unsigned short instanceID)
 {
 #ifdef PRINT_GC
-   cout << "delayedDeleteSocket: I=" << instanceID << endl;
+   std::cout << "delayedDeleteSocket: I=" << instanceID << std::endl;
 #endif
    ClosingSockets.insert(instanceID);
 }
@@ -360,26 +388,26 @@ void SCTPSocketMaster::delayedDeleteSocket(const unsigned short instanceID)
 void SCTPSocketMaster::socketGarbageCollection()
 {
 #ifdef PRINT_GC
-   cout << "Socket garbage collection..." << endl;
+   std::cout << "Socket garbage collection..." << std::endl;
 #endif
    MasterInstance.lock();
    LastGarbageCollection = getMicroTime();
 
    // ====== Try to auto-close connectionless associations ==================
-   multimap<int, SCTPSocket*>::iterator socketIterator = SocketList.begin();
+   std::multimap<int, SCTPSocket*>::iterator socketIterator = SocketList.begin();
    while(socketIterator != SCTPSocketMaster::SocketList.end()) {
       SCTPSocket* socket = socketIterator->second;
       socket->checkAutoClose();
       socketIterator++;
    }
 
-   // ====== Try to delete sockets already removed sockets using sctplib ====
-   set<int>::iterator iterator = ClosingSockets.begin();
+   // ====== Try to delete already removed sockets using sctplib ============
+   std::set<int>::iterator iterator = ClosingSockets.begin();
    while(iterator != ClosingSockets.end()) {
       const unsigned short instanceID = *iterator;
 
       bool used = false;
-      multimap<unsigned int, int>::iterator assocIterator = ClosingAssociations.begin();
+      std::multimap<unsigned int, int>::iterator assocIterator = ClosingAssociations.begin();
       while(assocIterator != ClosingAssociations.end()) {
          if(instanceID == assocIterator->second) {
             used = true;
@@ -390,11 +418,16 @@ void SCTPSocketMaster::socketGarbageCollection()
 
       if(!used) {
 #ifdef PRINT_GC
-         cout << "socketGarbageCollection: Removing instance #" << instanceID << "." << endl;
+         std::cout << "socketGarbageCollection: Removing instance #" << instanceID << "." << std::endl;
 #endif
          iterator++;
          ClosingSockets.erase(instanceID);
-         sctp_unregisterInstance(instanceID);
+         if(sctp_unregisterInstance(instanceID) != SCTP_SUCCESS) {
+#ifndef DISABLE_WARNINGS
+            std::cerr << "INTERNAL ERROR: SCTPSocketMaster::socketGarbageCollection() - sctp_unregisterInstance() failed!" << std::endl;
+#endif
+            ::abort();
+         }
       }
       else {
          iterator++;
@@ -403,8 +436,8 @@ void SCTPSocketMaster::socketGarbageCollection()
 
    MasterInstance.unlock();
 #ifdef PRINT_GC
-   cout << "Socket garbage collection completed in "
-        << getMicroTime() - LastGarbageCollection << " s" << endl;
+   std::cout << "Socket garbage collection completed in "
+             << getMicroTime() - LastGarbageCollection << " s" << std::endl;
 #endif
 }
 
@@ -414,11 +447,10 @@ bool SCTPSocketMaster::associationGarbageCollection(const unsigned int assocID,
                                                     const bool         sendAbort)
 {
    // ====== Delayed removal ================================================
-   multimap<unsigned int, int>::iterator iterator = ClosingAssociations.find(assocID);
+   std::multimap<unsigned int, int>::iterator iterator = ClosingAssociations.find(assocID);
    if(iterator != ClosingAssociations.end()) {
-      const unsigned short instanceID = iterator->second;
 #ifdef PRINT_GC
-      cout << "associationGarbageCollection: Removing association #" << assocID << "." << endl;
+      std::cout << "associationGarbageCollection: Removing association #" << assocID << "." << std::endl;
 #endif
 
       // ====== Delete association ==========================================
@@ -429,36 +461,15 @@ bool SCTPSocketMaster::associationGarbageCollection(const unsigned int assocID,
          sctp_abort(assocID);
 #endif
       }
-      sctp_deleteAssociation(assocID);
+      if(sctp_deleteAssociation(assocID) != SCTP_SUCCESS) {
+#ifndef DISABLE_WARNINGS
+         std::cerr << "INTERNAL ERROR: SCTPSocketMaster::associationGarbageCollection() - sctp_deleteAssociation() failed!" << std::endl;
+#endif
+         ::abort();
+      }
       ClosingAssociations.erase(iterator);
 
-      // ====== Check, if instance can be deleted ===========================
-      if(ClosingSockets.find(instanceID) != ClosingSockets.end()) {
-         // ====== Check, if instance is still in use =======================
-         bool deleteInstance = true;
-         iterator = ClosingAssociations.begin();
-         while(iterator != ClosingAssociations.end()) {
-            if(iterator->second == instanceID) {
-#ifdef PRINT_GC
-               cout << "associationGarbageCollection: Instance #" << instanceID
-                    << " is still in use -> no unregistering." << endl;
-#endif
-               deleteInstance = false;
-               break;
-            }
-            iterator++;
-         }
-
-         // ====== Delete instance ==========================================
-         if(deleteInstance) {
-#ifdef PRINT_GC
-            cout << "associationGarbageCollection: Removing instance #" << instanceID << "." << endl;
-#endif
-            ClosingSockets.erase(instanceID);
-            sctp_unregisterInstance(instanceID);
-         }
-      }
-
+      socketGarbageCollection();
       return(true);
    }
    return(false);
@@ -480,14 +491,14 @@ void SCTPSocketMaster::dataArriveNotif(unsigned int   assocID,
    snprintf((char*)&str,sizeof(str),
                "A%04d S%02d: Data Arrive Notification - length=%d, PPID=%u",
                assocID, streamID, length, protoID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
       SCTPNotification notification;
-      initNotification(notification,assocID,streamID);
+      initNotification(notification, assocID, streamID);
       sctp_data_arrive* sda = &notification.Content.sn_data_arrive;
       sda->sda_type          = SCTP_DATA_ARRIVE;
       sda->sda_flags         = (unordered == 1) ? SCTP_ARRIVE_UNORDERED : 0;
@@ -512,7 +523,7 @@ void SCTPSocketMaster::sendFailureNotif(unsigned int   assocID,
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Send Failure Notification",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
@@ -520,7 +531,7 @@ void SCTPSocketMaster::sendFailureNotif(unsigned int   assocID,
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
       SCTPNotification notification;
-      initNotification(notification,assocID,0);
+      initNotification(notification, assocID, 0);
       sctp_send_failed* ssf = &notification.Content.sn_send_failed;
       ssf->ssf_type     = SCTP_REMOTE_ERROR;
       ssf->ssf_flags    = 0;
@@ -560,7 +571,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Network Status Change",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
@@ -575,7 +586,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
 #endif
    if(ok != 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - sctp_getPathStatus() failed!" << endl;
+      std::cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - sctp_getPathStatus() failed!" << std::endl;
 #endif
       return;
    }
@@ -588,7 +599,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
                                       (char*)&pathStatus.destinationAddress);
    if(destination == NULL) {
 #ifndef DISABLE_WARNINGS
-      cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - Bad destination address!" << endl;
+      std::cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - Bad destination address!" << std::endl;
 #endif
       return;
    }
@@ -597,7 +608,7 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
       SCTPNotification notification;
-      initNotification(notification,assocID,0);
+      initNotification(notification, assocID, 0);
       sctp_paddr_change* spc = &notification.Content.sn_paddr_change;
       spc->spc_type     = SCTP_PEER_ADDR_CHANGE;
       spc->spc_flags    = 0;
@@ -676,20 +687,21 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
 #endif
                                              void*          dummy)
 {
+   SCTPAssociation* association = NULL;
+   SCTPNotification notification;
+   initNotification(notification, assocID, 0);
+
 #ifdef PRINT_NOTIFICATIONS
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Communication Up Notification",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
-      SCTPNotification notification;
-      initNotification(notification,assocID,0);
-
       // ====== Successfull associate() execution ===========================
-      SCTPAssociation* association = socket->getAssociationForAssociationID(assocID);
+      association = socket->getAssociationForAssociationID(assocID);
       if(association != NULL) {
          // ====== Correct RTOMax ============================================
          if(association->RTOMaxIsInitTimeout) {
@@ -699,7 +711,7 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
                char str[256];
                snprintf((char*)&str,sizeof(str),
                            "A%04d: Setting InitTimeout %d to saved RTOMax %d",assocID,status.rtoMax,association->RTOMax);
-               cerr << str << endl;
+               std::cerr << str << std::endl;
 #endif
                status.rtoMax = association->RTOMax;
                socket->setAssocStatus(assocID,status);
@@ -750,31 +762,41 @@ void* SCTPSocketMaster::communicationUpNotif(unsigned int   assocID,
       }
       // ====== Unwanted incoming association ===============================
       else {
+         std::cerr << "Incoming association, but not in listen mode -> rejecting association!" << std::endl;
+         association = NULL;
+      }
+   }
+
+   // ====== Generate "Communication Up" notification =======================
+   if(association != NULL) {
+      sctp_assoc_change* sac = &notification.Content.sn_assoc_change;
+      sac->sac_type   = SCTP_ASSOC_CHANGE;
+      sac->sac_flags  = 0;
+      sac->sac_length = sizeof(sctp_assoc_change);
+      sac->sac_state  = SCTP_COMM_UP;
+      sac->sac_error  = 0;
+      sac->sac_outbound_streams = noOfOutStreams;
+      sac->sac_inbound_streams  = noOfInStreams;
+      sac->sac_assoc_id         = assocID;
+      addNotification(socket,assocID,notification);
+   }
+   // ====== We do not want this new association -> remove it! ==============
+   else {
 #ifdef PRINT_NOTIFICATIONS
-         cerr << "Rejecting unwanted incoming association" << endl;
+      std::cerr << "Aborting and deleting unwanted incoming association!" << std::endl;
 #endif
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
-         sctp_abort(assocID, 0, NULL);
+      sctp_abort(assocID, 0, NULL);
 #elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
-         sctp_abort(assocID);
+      sctp_abort(assocID);
 #else
 #error Wrong sctplib version!
 #endif
-      }
-
-
-      // ====== Generate "Communication Up" notification ====================
-      if(association != NULL) {
-         sctp_assoc_change* sac = &notification.Content.sn_assoc_change;
-         sac->sac_type   = SCTP_ASSOC_CHANGE;
-         sac->sac_flags  = 0;
-         sac->sac_length = sizeof(sctp_assoc_change);
-         sac->sac_state  = SCTP_COMM_UP;
-         sac->sac_error  = 0;
-         sac->sac_outbound_streams = noOfOutStreams;
-         sac->sac_inbound_streams  = noOfInStreams;
-         sac->sac_assoc_id         = assocID;
-         addNotification(socket,assocID,notification);
+      if(sctp_deleteAssociation(assocID) != SCTP_SUCCESS) {
+#ifndef DISABLE_WARNINGS
+         std::cerr << "INTERNAL ERROR: SCTPSocketMaster::communicationUpNotif() - sctp_deleteAssociation() or rejected association failed!" << std::endl;
+#endif
+         ::abort();
       }
    }
 
@@ -799,7 +821,7 @@ void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Communication Lost Notification, Status=%d",assocID,status);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
@@ -821,7 +843,7 @@ void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
                char str[256];
                snprintf((char*)&str,sizeof(str),
                            "A%04d: Setting InitTimeout %d to saved RTOMax %d",assocID,status.rtoMax,association->RTOMax);
-               cerr << str << endl;
+               std::cerr << str << std::endl;
 #endif
                status.rtoMax = association->RTOMax;
                socket->setAssocStatus(assocID,status);
@@ -833,6 +855,7 @@ void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
          association->CommunicationLostNotification = true;
          association->ShutdownCompleteNotification  = true;
          association->ShutdownCompleteCondition.broadcast();
+         association->ReadUpdateCondition.broadcast();
 
 
          // ====== Generate "Communication Lost" notification ===============
@@ -879,7 +902,7 @@ void SCTPSocketMaster::communicationErrorNotif(unsigned int assocID,
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Communication Error Notification, Status=%d",assocID,status);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
@@ -887,7 +910,7 @@ void SCTPSocketMaster::communicationErrorNotif(unsigned int assocID,
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
       SCTPNotification notification;
-      initNotification(notification,assocID,0);
+      initNotification(notification, assocID, 0);
       sctp_remote_error* sre = &notification.Content.sn_remote_error;
       sre->sre_type      = SCTP_REMOTE_ERROR;
       sre->sre_flags     = 0;
@@ -906,7 +929,7 @@ void SCTPSocketMaster::restartNotif(unsigned int assocID, void* ulpDataPtr)
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Restart Notification",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
 
@@ -914,7 +937,7 @@ void SCTPSocketMaster::restartNotif(unsigned int assocID, void* ulpDataPtr)
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
       SCTPNotification notification;
-      initNotification(notification,assocID,0);
+      initNotification(notification);
       sctp_assoc_change* sac = &notification.Content.sn_assoc_change;
       sac->sac_type             = SCTP_ASSOC_CHANGE;
       sac->sac_flags            = 0;
@@ -928,7 +951,7 @@ void SCTPSocketMaster::restartNotif(unsigned int assocID, void* ulpDataPtr)
       }
       else {
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: SCTPSocketMaster::restartNotif() - sctp_getAssocStatus() failed!" << endl;
+         std::cerr << "WARNING: SCTPSocketMaster::restartNotif() - sctp_getAssocStatus() failed!" << std::endl;
 #endif
          sac->sac_outbound_streams = 1;
          sac->sac_inbound_streams  = 1;
@@ -946,12 +969,12 @@ void SCTPSocketMaster::shutdownReceivedNotif(unsigned int assocID, void* ulpData
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Shutdown Received Notification",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
    SCTPSocket* socket = getSocketForAssociationID(assocID);
    if(socket != NULL) {
-      SCTPAssociation* association = socket->getAssociationForAssociationID(assocID,false);
+      SCTPAssociation* association = socket->getAssociationForAssociationID(assocID, false);
       if(association != NULL) {
          // ====== Generate "Shutdown Complete" notification ================
          SCTPNotification notification;
@@ -974,7 +997,7 @@ void SCTPSocketMaster::shutdownCompleteNotif(unsigned int assocID, void* ulpData
    char str[256];
    snprintf((char*)&str,sizeof(str),
                "A%04d: Shutdown Complete Notification",assocID);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
    // ====== Delayed removal ================================================
@@ -993,6 +1016,7 @@ void SCTPSocketMaster::shutdownCompleteNotif(unsigned int assocID, void* ulpData
          association->ShutdownCompleteNotification = true;
          association->ShutdownCompleteCondition.broadcast();
          association->ReadyForTransmit.broadcast();
+         association->ReadUpdateCondition.broadcast();
 
          // ====== Generate "Shutdown Complete" notification ================
          SCTPNotification notification;
@@ -1008,8 +1032,8 @@ void SCTPSocketMaster::shutdownCompleteNotif(unsigned int assocID, void* ulpData
          sac->sac_assoc_id         = assocID;
          addNotification(socket,assocID,notification);
       }
+      socket->checkAutoClose();
    }
-   socket->checkAutoClose();
 }
 
 
@@ -1025,7 +1049,7 @@ void SCTPSocketMaster::queueStatusChangeNotif(unsigned int assocID,
    snprintf((char*)&str,sizeof(str),
                "A%04d: Queue Status Change Notification, Qid=%d Qtype=%d Qlength=%d",
                assocID,queueIdentifier,queueType,queueLength);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
    SCTPSocket* socket = getSocketForAssociationID(assocID);
@@ -1053,7 +1077,7 @@ void SCTPSocketMaster::asconfStatusNotif(unsigned int assocID,
    snprintf((char*)&str,sizeof(str),
                "A%04d: ASConf Status Change Notification, Cid=%d result=%d",
                assocID,correlationID,result);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 }
 #endif
@@ -1066,11 +1090,11 @@ void SCTPSocketMaster::userCallback(int        fileDescriptor,
                                     void*      userData)
 {
    char str[256];
-#ifdef PRINT_NOTIFICATIONS
+#ifdef PRINT_USERCALLBACK
    snprintf((char*)&str,sizeof(str),
                "F%04d: User Callback, mask=$%x",
                fileDescriptor,eventMask);
-   cerr << str << endl;
+   std::cerr << str << std::endl;
 #endif
 
    UserSocketNotification* usn = (UserSocketNotification*)userData;
@@ -1084,12 +1108,12 @@ void SCTPSocketMaster::userCallback(int        fileDescriptor,
       }
       else {
 #ifdef PRINT_PIPE
-         cout << getpid() << ": Break via break pipe received" << endl;
+         std::cout << getpid() << ": Break via break pipe received" << std::endl;
 #endif
          ssize_t received = read(BreakPipe[0],(char*)&str,sizeof(str));
          while(received > 0) {
 #ifdef PRINT_PIPE
-           cout << "Break Pipe: " << received << " calls" << endl;
+           std::cout << "Break Pipe: " << received << " calls" << std::endl;
 #endif
            received = read(BreakPipe[0],(char*)&str,sizeof(str));
          }
@@ -1112,7 +1136,7 @@ SCTPSocket* SCTPSocketMaster::getSocketForAssociationID(const unsigned int assoc
 
    if(sctp_getInstanceID(assocID,&instanceID) == 0) {
       if(instanceID != 0) {
-         multimap<int, SCTPSocket*>::iterator iterator =
+         std::multimap<int, SCTPSocket*>::iterator iterator =
             SocketList.find((int)instanceID);
          if(iterator != SocketList.end()) {
             return(iterator->second);
@@ -1120,15 +1144,15 @@ SCTPSocket* SCTPSocketMaster::getSocketForAssociationID(const unsigned int assoc
       }
 #ifdef DEBUG
       else {
-        cerr << "WARNING: SCTPSocketMaster::getSocketForAssociationID() - "
-                "Instance ID for association ID #" << assocID << " is zero!" << endl;
+        std::cerr << "WARNING: SCTPSocketMaster::getSocketForAssociationID() - "
+                     "Instance ID for association ID #" << assocID << " is zero!" << std::endl;
       }
 #endif
    }
 #ifdef DEBUG
    else {
-      cerr << "WARNING: SCTPSocketMaster::getSocketForAssociationID() - "
-              "No instance ID for association ID #" << assocID << " found!" << endl;
+      std::cerr << "WARNING: SCTPSocketMaster::getSocketForAssociationID() - "
+                   "No instance ID for association ID #" << assocID << " found!" << std::endl;
    }
 #endif
    return(NULL);
@@ -1156,14 +1180,14 @@ bool SCTPSocketMaster::initNotification(SCTPNotification& notification,
    notification.Content.sn_header.sn_type = SCTP_UNDEFINED;
    notification.ContentPosition = 0;
    SCTP_Association_Status status;
-   if(sctp_getAssocStatus(assocID,&status) == 0) {
+   if(sctp_getAssocStatus(assocID, &status) == 0) {
       notification.RemotePort      = status.destPort;
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19) || (SCTPLIB_VERSION == SCTPLIB_1_0_0)
-      notification.RemoteAddresses = min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
-                                         status.numberOfAddresses);
+      notification.RemoteAddresses = std::min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
+                                              status.numberOfAddresses);
 #elif (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE20) || (SCTPLIB_VERSION == SCTPLIB_1_3_0)
-      notification.RemoteAddresses = min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
-                                         status.numberOfDestinationPaths);
+      notification.RemoteAddresses = std::min((unsigned short)SCTP_MAX_NUM_ADDRESSES,
+                                              status.numberOfDestinationPaths);
 #else
 #error Wrong sctplib version!
 #endif
@@ -1178,8 +1202,8 @@ bool SCTPSocketMaster::initNotification(SCTPNotification& notification,
 #endif
 
 #ifndef DISABLE_WARNINGS
-            cerr << "WARNING: SCTPSocketMaster::initNotification() - sctp_getPathStatus() failure!"
-                 << endl;
+            std::cerr << "WARNING: SCTPSocketMaster::initNotification() - sctp_getPathStatus() failure!"
+                      << std::endl;
 #endif
 
          }
@@ -1192,8 +1216,8 @@ bool SCTPSocketMaster::initNotification(SCTPNotification& notification,
       return(true);
    }
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: SCTPSocketMaster::initNotification() - sctp_getAssocStatus() failure!"
-              << endl;
+         std::cerr << "WARNING: SCTPSocketMaster::initNotification() - sctp_getAssocStatus() failure!"
+                   << std::endl;
 #endif
    return(false);
 }
@@ -1220,14 +1244,15 @@ void SCTPSocketMaster::addNotification(SCTPSocket*             socket,
       ((notification.Content.sn_header.sn_type == SCTP_SEND_FAILED)      && (notificationFlags & SCTP_RECVSENDFAILEVNT)) ||
       ((notification.Content.sn_header.sn_type == SCTP_SHUTDOWN_EVENT)   && (notificationFlags & SCTP_RECVSHUTDOWNEVNT))) {
 
-      // ====== Add notification to global or association's queue ===========
-#ifdef PRINT_ASSOC_USECOUNT
-      cout << "AddNotification: UseCount increment for A" << association->getID() << ": "
-           << association->UseCount << " -> ";
-#endif
       association->UseCount++;
 #ifdef PRINT_ASSOC_USECOUNT
-      cout << association->UseCount << ". Notification Type = " << notification.Content.sn_header.sn_type << endl;
+      std::cout << association->UseCount << ". Notification Type = " << notification.Content.sn_header.sn_type << std::endl;
+#endif
+
+      // ====== Add notification to global or association's queue ===========
+#ifdef PRINT_ASSOC_USECOUNT
+      std::cout << "AddNotification: UseCount increment for A" << association->getID() << ": "
+                << association->UseCount << " -> ";
 #endif
       if( (socket->Flags & SCTPSocket::SSF_GlobalQueue) &&
           (association->PeeledOff == false) ) {
@@ -1239,6 +1264,16 @@ void SCTPSocketMaster::addNotification(SCTPSocket*             socket,
          association->ReadReady = association->hasData();
       }
    }
+
+   else {
+      /* The user does not want a notification, but an action has to be
+         signallised in order to end waiting for events (e.g. interrupting
+         SCTPSocket::interalReceive() in case of a shutdown complete)! */
+      if(!((socket->Flags & SCTPSocket::SSF_GlobalQueue) &&
+           (association->PeeledOff == false))) {
+         association->InQueue.signal();
+      }
+   }
 }
 
 
@@ -1251,16 +1286,16 @@ void SCTPSocketMaster::addUserSocketNotification(UserSocketNotification* usn)
 
    if(result < 0) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SCTPSocketMaster::addUserSocketNotification() - sctp_registerUserCallback() failed!" << endl;
+      std::cerr << "ERROR: SCTPSocketMaster::addUserSocketNotification() - sctp_registerUserCallback() failed!" << std::endl;
 #endif
    }
 
    if((usn->FileDescriptor != BreakPipe[0]) && (BreakPipe[0] != -1)) {
       char dummy = 'T';
 #ifdef PRINT_PIPE
-      cout << "Sending Break via break pipe..." << endl;
+      std::cout << "Sending Break via break pipe..." << std::endl;
 #endif
-      write(BreakPipe[1],&dummy,sizeof(dummy));
+      dummy = write(BreakPipe[1],&dummy,sizeof(dummy));
    }
 
    unlock();
@@ -1271,7 +1306,11 @@ void SCTPSocketMaster::addUserSocketNotification(UserSocketNotification* usn)
 void SCTPSocketMaster::deleteUserSocketNotification(UserSocketNotification* usn)
 {
    lock();
-   sctp_unregisterUserCallback(usn->FileDescriptor);
+   if(sctp_unregisterUserCallback(usn->FileDescriptor) != SCTP_SUCCESS) {
+#ifndef DISABLE_WARNINGS
+      std::cerr << "INTERNAL ERROR: SCTPSocketMaster::deleteUserSocketNotification() - sctp_unregisterUserCallback() failed!" << std::endl;
+#endif
+   }
    unlock();
 }
 
