@@ -1,33 +1,33 @@
 /*
- *  $Id: socketaddress.cc 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Socket Address Implementation
@@ -58,7 +58,7 @@ SocketAddress** SocketAddress::newAddressList(const cardinal entries)
    SocketAddress** list = new SocketAddress*[entries + 1];
    if(list == NULL) {
 #ifndef DISABLE_WARNINGS
-      cerr << "SocketAddress::newAddressList() - Out of memory!" << endl;
+      std::cerr << "SocketAddress::newAddressList() - Out of memory!" << std::endl;
 #endif
       return(NULL);
    }
@@ -93,7 +93,7 @@ SocketAddress* SocketAddress::getLocalAddress(const SocketAddress& peer)
    SocketAddress* address = createSocketAddress(family);
    if(address != NULL) {
       int sd = ext_socket(family,SOCK_DGRAM,0);
-      if(socket >= 0) {
+      if(sd >= 0) {
          sockaddr_storage socketAddress;
          socklen_t        socketAddressLength =
                              peer.getSystemAddress((sockaddr*)&socketAddress,SocketAddress::MaxSockLen,
@@ -127,8 +127,8 @@ SocketAddress* SocketAddress::createSocketAddress(const integer family)
        break;
       default:
 #ifndef DISABLE_WARNINGS
-          cerr << "ERROR: SocketAddress::createSocketAddress(family) - "
-                  "Unknown address family " << family << "!" << endl;
+          std::cerr << "ERROR: SocketAddress::createSocketAddress(family) - "
+                       "Unknown address family " << family << "!" << std::endl;
 #endif
        break;
    }
@@ -150,7 +150,7 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal flags,
    }
    if(internetAddress == NULL) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SocketAddress::createSocketAddress(name) - Out of memory!" << endl;
+      std::cerr << "ERROR: SocketAddress::createSocketAddress(name) - Out of memory!" << std::endl;
 #endif
    }
    if(internetAddress->isValid()) {
@@ -163,7 +163,7 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal flags,
    UnixAddress* unixAddress = new UnixAddress(name);
    if(unixAddress == NULL) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SocketAddress::createSocketAddress(name) - Out of memory!" << endl;
+      std::cerr << "ERROR: SocketAddress::createSocketAddress(name) - Out of memory!" << std::endl;
 #endif
    }
    if(unixAddress->isValid()) {
@@ -185,7 +185,7 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal flags,
    InternetAddress* internetAddress = new InternetAddress(name,port);
    if(internetAddress == NULL) {
 #ifndef DISABLE_WARNINGS
-      cerr << "ERROR: SocketAddress::createSocketAddress(name,port) - Out of memory!" << endl;
+      std::cerr << "ERROR: SocketAddress::createSocketAddress(name,port) - Out of memory!" << std::endl;
 #endif
    }
    if(internetAddress->isValid()) {
@@ -208,7 +208,7 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal  flags,
             InternetAddress* internetAddress = new InternetAddress(address,length);
             if(internetAddress == NULL) {
 #ifndef DISABLE_WARNINGS
-               cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - Out of memory!" << endl;
+               std::cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - Out of memory!" << std::endl;
 #endif
             }
             if(internetAddress->isValid()) {
@@ -221,7 +221,7 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal  flags,
             UnixAddress* unixAddress = new UnixAddress(address,length);
             if(unixAddress == NULL) {
 #ifndef DISABLE_WARNINGS
-               cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - Out of memory!" << endl;
+               std::cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - Out of memory!" << std::endl;
 #endif
             }
             if(unixAddress->isValid()) {
@@ -232,8 +232,8 @@ SocketAddress* SocketAddress::createSocketAddress(const cardinal  flags,
        break;
       default:
 #ifndef DISABLE_WARNINGS
-          cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - "
-                  "Unknown address family " << address->sa_family << "!" << endl;
+          std::cerr << "ERROR: SocketAddress::createSocketAddress(sockaddr) - "
+                       "Unknown address family " << address->sa_family << "!" << std::endl;
 #endif
        break;
    }
