@@ -1,33 +1,33 @@
 /*
- *  $Id: socketaddress.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Socket Adress
@@ -40,6 +40,7 @@
 
 
 #include "tdsystem.h"
+#include "tdstrings.h"
 
 
 #include <sys/socket.h>
@@ -51,9 +52,9 @@
   * This class is an interface for a socket address.
   *
   * @short   Socket Address
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
-  */            
+  */
 class SocketAddress
 {
    // ====== Destructor =====================================================
@@ -239,7 +240,7 @@ class SocketAddress
      * @param length Length of buffer.
      * @param type Socket address type, e.g. AF_INET or AF_INET6.
      * @return Length of written sockaddr structure.
-     */            
+     */
    virtual cardinal getSystemAddress(sockaddr*       buffer,
                                      const socklen_t length,
                                      const cardinal  type = AF_UNSPEC) const = 0;
@@ -250,7 +251,7 @@ class SocketAddress
      * @param address sockaddr.
      * @param length Length of sockaddr.
      */
-   virtual bool setSystemAddress(sockaddr* address, const socklen_t length) = 0;
+   virtual bool setSystemAddress(const sockaddr* address, const socklen_t length) = 0;
 
 
    // ====== SocketAddress lists ============================================
@@ -289,14 +290,14 @@ class SocketAddress
    cardinal Format;
 
 
-   friend inline ostream& operator<<(ostream& os, const SocketAddress& sa);
+   friend inline std::ostream& operator<<(std::ostream& os, const SocketAddress& sa);
 };
 
 
 /**
   * Output operator.
   */
-inline ostream& operator<<(ostream& os, const SocketAddress& sa);
+inline std::ostream& operator<<(std::ostream& os, const SocketAddress& sa);
 
 
 #include "socketaddress.icc"
