@@ -1,33 +1,33 @@
 /*
- *  $Id: tdmessage.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Socket Message
@@ -49,6 +49,48 @@
 #include <sys/socket.h>
 
 
+/**
+  * Wrapper for CMSG_SPACE macro.
+  */
+inline static size_t CSpace(const size_t payloadLength);
+
+/**
+  * Wrapper for CMSG_LEN macro.
+  */
+inline static size_t CLength(const size_t payloadLength);
+
+
+/**
+  * Wrapper for CMSG_DATA macro.
+  */
+inline static void* CData(cmsghdr* cmsg);
+
+/**
+  * Wrapper for CMSG_FIRSTHDR macro.
+  */
+inline static cmsghdr* CFirstHeader(msghdr* header);
+
+/**
+  * Wrapper for CMSG_NXTHDR macro.
+  */
+inline static cmsghdr* CNextHeader(msghdr* header, const cmsghdr* cmsg);
+
+
+/**
+  * Wrapper for CMSG_DATA macro (constant version).
+  */
+inline static const void* CData(const cmsghdr* cmsg);
+
+/**
+  * Wrapper for CMSG_FIRSTHDR macro (constant version).
+  */
+inline static const cmsghdr* CFirstHeader(const msghdr* header);
+
+/**
+  * Wrapper for CMSG_NXTHDR macro (constant version).
+  */
+inline static const cmsghdr* CNextHeader(const msghdr* header, const cmsghdr* cmsg);
+
 
 /**
   * This template class manages manages message structures used by
@@ -56,7 +98,7 @@
   * of the control data block.
   *
   * @short   Socket Message
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   */
 template<const size_t size> class SocketMessage
@@ -87,8 +129,10 @@ template<const size_t size> class SocketMessage
      * Set address.
      *
      * @param address SocketAddress object.
+     * @param type Address type (AF_UNSPEC to take default from address).
      */
-   inline void setAddress(const SocketAddress& address);
+   inline void setAddress(const SocketAddress& address,
+                          const integer        family = AF_UNSPEC);
 
    /**
      * Set buffer.
@@ -96,35 +140,28 @@ template<const size_t size> class SocketMessage
      * @param buffer Buffer.
      * @param bufferSize Size of buffer.
      */
-   inline void setBuffer(char* buffer, const size_t buffersize);
-
-   /**
-     * Set size of control block. Sizes greater than the template parameter
-     * are adjusted to the maximum possible value.
-     *
-     * @param controlsize Size of controlblock.
-     */
-   inline void setControl(const size_t controlsize);
+   inline void setBuffer(void* buffer, const size_t buffersize);
 
    /**
      * Add control header of given cmsg level and type. Returns NULL,
      * if there is not enough free space in the control data block.
+     * The new control header is cleared (i.e. all bytes set to 0).
      *
      * @param payload Size of payload.
      * @param level Level (e.g. IPPROTO_SCTP).
      * @param type Type (e.g. SCTP_INIT).
      * @return Pointer to begin of *payload* area.
      */
-   inline char* addHeader(const cardinal payload,
-                          const int      level,
-                          const int      type);
+   inline void* addHeader(const size_t payloadLength,
+                          const int    level,
+                          const int    type);
 
    /**
      * Get first cmsg header in control block.
      *
      * @return First cmsg header or NULL, if there are none.
      */
-   inline cmsghdr* getFirstHeader() const;
+   inline cmsghdr* getFirstHeader();
 
    /**
      * Get next cmsg header in control block.
@@ -132,8 +169,21 @@ template<const size_t size> class SocketMessage
      * @param prev Previous cmsg header.
      * @return Next cmsg header or NULL, if there are no more.
      */
-   inline cmsghdr* getNextHeader(const cmsghdr* prev) const;
+   inline cmsghdr* getNextHeader(cmsghdr* prev);
 
+   /**
+     * Get flags.
+     *
+     * @return Flags.
+     */
+   inline int getFlags() const;
+
+   /**
+     * Set flags.
+     *
+     * @param flags Flags.
+     */
+   inline void setFlags(const int flags);
 
    // ====== Message structures =============================================
    /**
@@ -158,45 +208,14 @@ template<const size_t size> class SocketMessage
      * Control data block, its size is given by the template parameter.
      */
    public:
-   char Control[size];
+#ifdef SOLARIS
+   char Control[_CMSG_DATA_ALIGN(sizeof(struct cmsghdr)) + _CMSG_DATA_ALIGN(size)];
+#else
+   char Control[CMSG_SPACE(size)];
+#endif
 };
 
 
-/**
-  * Wrapper for CMSG_SPACE macro.
-  */
-#if (SYSTEM == OS_SOLARIS)
-#define CSpace(payload) (_CMSG_DATA_ALIGN(sizeof(struct cmsghdr)) + _CMSG_DATA_ALIGN(payload))
-#else
-#define CSpace(payload) CMSG_SPACE(payload)
-#endif
-
-/**
-  * Wrapper for CMSG_LEN macro.
-  */
-
-#if (SYSTEM == OS_SOLARIS)
-#define CLength(l) (_CMSG_DATA_ALIGN(sizeof(struct cmsghdr)) + (l))
-#else
-#define CLength(l) CMSG_LEN(l)
-#endif
-
-/**
-  * Wrapper for CMSG_DATA macro.
-  */
-inline static char* CData(const cmsghdr* cmsg);
-
-/**
-  * Wrapper for CMSG_FIRSTHDR macro.
-  */
-inline static cmsghdr* CFirst(const msghdr* header);
-
-/**
-  * Wrapper for CMSG_NXTHDR macro.
-  */
-inline static cmsghdr* CNext(const msghdr* header, const cmsghdr* cmsg);
-
-
 #include "tdmessage.icc"
 
 
