@@ -1,33 +1,33 @@
 /*
- *  $Id: tdstrings.cc 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Strings Implementation
@@ -71,7 +71,7 @@ String::String(const char* string, const cardinal length)
       memcpy((void*)&str,string,length);
       str[length] = 0x00;
       setData(stringDuplicate((char*)&str));
-   }   
+   }
    else {
       setData(NULL);
    }
@@ -82,7 +82,7 @@ String::String(const char* string, const cardinal length)
 String::String(const cardinal value)
 {
    char string[64];
-   snprintf((char*)&string,sizeof(string),"%Ld",(card64)value);
+   snprintf((char*)&string,sizeof(string),"%llu",(unsigned long long int)value);
    setData(stringDuplicate((char*)&string));
 }
 
@@ -158,7 +158,7 @@ String String::toUpper() const
 // ###### Get left part of string ###########################################
 String String::left(const cardinal maxChars) const
 {
-   const cardinal len = min(length(),maxChars);
+   const cardinal len = std::min(length(),maxChars);
    char buffer[len + 1];
 
    cardinal i;
@@ -178,7 +178,7 @@ String String::mid(const cardinal start, const cardinal maxChars) const
       return("");
    }
 
-   const cardinal len = min(strlen - start,maxChars);
+   const cardinal len = std::min(strlen - start,maxChars);
    char buffer[len + 1];
 
    cardinal i;
@@ -194,7 +194,7 @@ String String::mid(const cardinal start, const cardinal maxChars) const
 String String::right(const cardinal maxChars) const
 {
    const cardinal strlen = length();
-   const cardinal len    = min(strlen,maxChars);
+   const cardinal len    = std::min(strlen,maxChars);
    char buffer[len + 1];
 
    cardinal i,j;
@@ -243,7 +243,7 @@ bool String::scanSetting(String& s1, String& s2) const
 
 
 // ###### "<<"-operator #####################################################
-ostream& operator<<(ostream& os, const String& string)
+std::ostream& operator<<(std::ostream& os, const String& string)
 {
    const char* data = string.getData();
    if(data) {
