@@ -1,33 +1,33 @@
 /*
- *  $Id: thread.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Thread Implementation
@@ -58,7 +58,7 @@
   * section "Async Signal Safety" for more information!
   *
   * @short   Thread
-  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
+  * @author  Thomas Dreibholz (thomas.dreibholz@gmail.com)
   * @version 1.0
   * @see Synchronizable
   */
@@ -86,7 +86,7 @@ class Thread : public Synchronizable
      */
    Thread(const char*    name  = "Thread",
           const cardinal flags = TF_CancelDeferred);
-   
+
    /**
      * Destructor. The thread will be stopped (if running) and deleted.
      */
@@ -106,7 +106,7 @@ class Thread : public Synchronizable
      */
    inline pid_t getPID() const;
 
-   
+
    // ====== Delay ==========================================================
    /**
      * Delay execution of current thread for a given timeout.
@@ -118,7 +118,7 @@ class Thread : public Synchronizable
      */
    static card64 delay(const card64 delayTimeout, const bool interruptable = false);
 
-   
+
    // ====== Thread control =================================================
    /**
      * Start the thread, if not already started.
@@ -127,7 +127,7 @@ class Thread : public Synchronizable
      * @return true, if the thread has been started; false, if not.
      */
    virtual bool start(const char* name = NULL);
-   
+
    /**
      * Stop the thread, if not already stopped.
      * If the thread flag ThreadCancelAsynchronous is set, it will be stopped
@@ -139,7 +139,7 @@ class Thread : public Synchronizable
      * @return Return value from stopped thread.
      */
    virtual void* stop();
-   
+
    /**
      * Wait for the thread to be finished.
      *
@@ -165,27 +165,6 @@ class Thread : public Synchronizable
    inline static cardinal setCancelState(const cardinal state);
 
 
-   // ====== Public data ====================================================
-   public:
-   /**
-     * Mutex for set of Synchronizable objects (for SYNCDEBUGGER mode only!).
-     *
-     * @see Synchronizable
-     */
-   static Synchronizable SyncSetLock;
-
-   /**
-     * This mutex is necessary to avoid simultaneous access to malloc()
-     * and free() when using libefence.
-     */
-   static Synchronizable MemoryManagementLock;
-
-   /**
-     * Set of all running threads.
-     */
-   static set<Thread*> ThreadSet;
-
-
    // ====== Tests for cancellation =========================================
    protected:
    /**
@@ -232,41 +211,6 @@ class Thread : public Synchronizable
    cardinal        Flags;
    pthread_mutex_t StartupMutex;
    pthread_cond_t  StartupCondition;
-
-
-#ifdef SYNCDEBUGGER
-   friend class Synchronizable;
-   struct _pthread_descr_struct;
-   typedef struct _pthread_descr_struct* pthread_descr;
-
-   // ***********************************************************************
-   // * IMPORTANT: Insert _pthread_descr_struct from
-   // *            linuxthreads/internals.h of your glibc package here!
-   // ***********************************************************************
-   struct _pthread_descr_struct {
-#if ((__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 2))
-     union {
-       struct {
-         Thread::pthread_descr self;	/* Pointer to this structure */
-       } data;
-       void *__padding[16];
-     } p_header;
-#endif
-     pthread_descr p_nextlive, p_prevlive;
-                                   /* Double chaining of active threads */
-     pthread_descr p_nextwaiting;  /* Next element in the queue holding the thr */
-     pthread_descr p_nextlock;	/* can be on a queue and waiting on a lock */
-     pthread_t p_tid;              /* Thread identifier */
-     int p_pid;                    /* PID of Unix process */
-   };
-   // ***********************************************************************
-
-   pthread_descr InternalPThreadPtr;
-
-
-   static bool checkSyncDebugger();
-   static bool syncDebuggerChecked;
-#endif
 };
 
 
