@@ -1,33 +1,33 @@
 /*
- *  $Id: tools.cc 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Tools Implementation
@@ -108,7 +108,7 @@ bool scanURL(const String& location,
       return(false);
    }
    host = url.left(p2);
-   
+
    // ====== Get path =======================================================
    path = url.mid(p2 + 1);
 
@@ -119,7 +119,7 @@ bool scanURL(const String& location,
 
 
 // ###### Print time stamp ##################################################
-void printTimeStamp(ostream& os)
+void printTimeStamp(std::ostream& os)
 {
    char str[128];
    const card64 microTime = getMicroTime();
@@ -163,76 +163,3 @@ bool getUserName(char*        str,
    str[0] = 0x00;
    return(false);
 }
-
-
-#ifdef USE_EFENCE
-
-
-// VERY IMPORTANT:
-// 1. malloc() and free() have to be synchronized in order to get correct
-//    results!!!
-// 2. new() and delete() are replaced by compatible versions.
-//    This does *not* include the C-functions malloc() and free()!
-//    C sources using malloc() and free() require additional care!
-
-
-#include "thread.h"
-
-
-// Allocation balance to detect memory leaks.
-static int64 AllocationBalance = 0;
-
-
-// ###### Operator new() replacement for libefence usage ####################
-void* operator new(size_t size) throw (std::bad_alloc)
-{
-   const cardinal oldstate = Thread::setCancelState(Thread::TCS_CancelDisabled);
-   Thread::MemoryManagementLock.synchronized();
-
-   if(size <= 0) {
-      size = 1;
-#ifndef DISABLE_WARNINGS
-      cerr << "WARNING: operator new() - Size is 0!" << endl;
-#endif
-   }
-   void* ptr = malloc(size + 4);
-   if(ptr != NULL) {
-      card32* ptr32 = (card32*)ptr;
-      *ptr32 = (card32)size;
-      ptr = (void*)((long)ptr + (long)4);
-      AllocationBalance += (int64)size;
-
-#ifdef PRINT_ALLOCATIONS
-   cout << "Allocated " << size << " bytes -> Balance is "
-        << AllocationBalance << "." << endl;
-#endif
-   }
-
-   Thread::MemoryManagementLock.unsynchronized();
-   Thread::setCancelState(oldstate);
-   return(ptr);
-}
-
-
-// ###### Operator delete() replacement for libefence usage #################
-void operator delete(void* ptr) throw ()
-{
-   const cardinal oldstate = Thread::setCancelState(Thread::TCS_CancelDisabled);
-   Thread::MemoryManagementLock.synchronized();
-
-   card32* ptr32 = (card32*)((long)ptr - (long)4);
-   card32 size = *ptr32;
-   AllocationBalance -= (int64)size;
-   free(ptr32);
-
-#ifdef PRINT_ALLOCATIONS
-   cout << "Freed " << size << " bytes -> Balance is "
-        << AllocationBalance << "." << endl;
-#endif
-
-   Thread::MemoryManagementLock.unsynchronized();
-   Thread::setCancelState(oldstate);
-}
-
-
-#endif
