@@ -1,33 +1,33 @@
 /*
- *  $Id: tools.h 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Tools Implementation
@@ -67,6 +67,47 @@ inline void debug(const char* string);
 inline card64 getMicroTime();
 
 
+/**
+  * Translate 16-bit value to network byte order.
+  *
+  * @param x Value to be translated.
+  * @return Translated value.
+  */
+inline card16 translate16(const card16 x);
+
+/**
+  * Translate 32-bit value to network byte order.
+  *
+  * @param x Value to be translated.
+  * @return Translated value.
+  */
+inline card32 translate32(const card32 x);
+
+/**
+  * Translate 64-bit value to network byte order.
+  *
+  * @param x Value to be translated.
+  * @return Translated value.
+  */
+inline card64 translate64(const card64 x);
+
+/**
+  * Translate double to 64-bit binary.
+  *
+  * @param x Value to be translated.
+  * @return Translated value.
+  */
+inline card64 translateToBinary(const double x);
+
+/**
+  * Translate 64-bit binary to double.
+  *
+  * @param x Value to be translated.
+  * @return Translated value.
+  */
+inline double translateToDouble(const card64 x);
+
+
 /**
   * Calculate packets per second.
   *
@@ -188,35 +229,10 @@ template<class T> cardinal removeDuplicates(T*             array,
   *
   * @param os Output stream.
   */
-void printTimeStamp(ostream& os = cout);
-
-
-#ifdef USE_EFENCE
-
-
-#include "synchronizable.h"
-
-
-extern Synchronizable MemoryManagementLock;
-
-
-/**
-  * operator new() replacement for usage with libefence.
-  */ 
-void* operator new(size_t size) throw (std::bad_alloc);
-
-
-/**
-  * operator delete() replacement for usage with libefence.
-  */ 
-void operator delete(void* ptr) throw ();
-
-
-#endif
+void printTimeStamp(std::ostream& os = std::cout);
 
 
 #include "tools.icc"
 
 
 #endif
-
