@@ -1,33 +1,33 @@
 /*
- *  $Id: unixaddress.cc 183 2003-08-19 19:28:34Z tuexen $
  *
  * SocketAPI implementation for the sctplib.
- * Copyright (C) 1999-2003 by Thomas Dreibholz
+ * Copyright (C) 1999-2026 by Thomas Dreibholz
  *
  * Realized in co-operation between
  * - Siemens AG
- * - University of Essen, Institute of Computer Networking Technology
- * - University of Applied Sciences, Muenster
+ * - University of Duisburg-Essen, Institute for Experimental Mathematics
+ * - Münster University of Applied Sciences, Burgsteinfurt
  *
  * Acknowledgement
  * This work was partially funded by the Bundesministerium fuer Bildung und
  * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact: discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
  *
  * Purpose: Unix address implementation
@@ -62,13 +62,13 @@ UnixAddress::UnixAddress(const UnixAddress& address)
 
 
 // ###### Unix address constructor ######################################
-UnixAddress::~UnixAddress() 
+UnixAddress::~UnixAddress()
 {
 }
 
 
 // ###### Unix address constructor ######################################
-UnixAddress::UnixAddress(sockaddr* address, const cardinal length)
+UnixAddress::UnixAddress(const sockaddr* address, const cardinal length)
 {
    setSystemAddress(address,length);
 }
@@ -95,7 +95,7 @@ void UnixAddress::init(const String& name)
    }
    else {
 #ifndef DISABLE_WARNINGS
-      cerr << "WARNING: UnixAddress::init() - Name too long!" << endl;
+      std::cerr << "WARNING: UnixAddress::init() - Name too long!" << std::endl;
 #endif
    }
 }
@@ -170,16 +170,16 @@ cardinal UnixAddress::getSystemAddress(sockaddr*       buffer,
          }
          else {
 #ifndef DISABLE_WARNINGS
-            cerr << "WARNING: UnixAddress::getSystemUnixAddress() - "
-                    "Buffer size too low for AF_UNIX!" << endl;
+            std::cerr << "WARNING: UnixAddress::getSystemUnixAddress() - "
+                         "Buffer size too low for AF_UNIX!" << std::endl;
 #endif
          }
         }
        break;
       default:
 #ifndef DISABLE_WARNINGS
-         cerr << "WARNING: UnixAddress::getSystemUnixAddress() - Unknown type "
-              << type << "!" << endl;
+         std::cerr << "WARNING: UnixAddress::getSystemUnixAddress() - Unknown type "
+                   << type << "!" << std::endl;
 #endif
        break;
    }
@@ -188,7 +188,7 @@ cardinal UnixAddress::getSystemAddress(sockaddr*       buffer,
 
 
 // ###### Initialize internet address from sockaddr structure ###############
-bool UnixAddress::setSystemAddress(sockaddr* address, const socklen_t length)
+bool UnixAddress::setSystemAddress(const sockaddr* address, const socklen_t length)
 {
    sockaddr_un* unixAddress = (sockaddr_un*)address;
    switch(unixAddress->sun_family) {
